#!/bin/bash

#==============================================================================
# CheckMySQLConnection.sh
#==============================================================================

BN=${0##*/}

usage() {
  echo "Usage:"
  echo "  $BN DB"
  echo "Description:"
  echo "  returns success if \$EPAGES_CONFIG/Database.d/DB.conf can be connected"
  echo "  returns failue otherwise"
  exit 1
}

# show usage if not exactly 1 argument or argument starts with - (like -?)

[[ $# != 1 || $1 != ${1#-} ]] && usage

# failure if mysql program does not exist

type mysql >/dev/null || exit 1

# try to source /etc/default/epages6

SYS_SCRIPT=/etc/sysconfig/epages6
[[ -f $SYS_SCRIPT ]] && . $SYS_SCRIPT
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
RUN_SCRIPT=/etc/default/$EPAGES_PROJECT
[[ -f $RUN_SCRIPT ]] && . $RUN_SCRIPT

# failure if $EPAGES_CONFIG/Database.d/$1.conf is missing

DB_CONF=${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}/Database.d/$1.conf
ls $DB_CONF >/dev/null || exit 1

# failure if /var/epages/ini-config.sh is missing

INI_CONFIG=/var/epages/ini-config.sh
ls $INI_CONFIG >/dev/null || exit 1

# get connection from $EPAGES_CONFIG/Databse.d/$1.conf

database=$($INI_CONFIG -f $DB_CONF -s Connection -k Database -G)
login=$($INI_CONFIG -f $DB_CONF -s Connection -k login -G)

# failure if Database= or login= is missing/empty

[[ -z $database ]] && { echo "$DB_CONF: Database= missing/empty"; exit 1; }
[[ -z $login ]] && { echo "$DB_CONF: login= missing/empty"; exit 1; }

# datasource has 3 parts divided by : (e.g.: datasource=dbi:mysql:host=localhost)

datasource=$($INI_CONFIG -f $DB_CONF -s Connection -k datasource -G)
datasource=(${datasource//:/ })
host=${datasource[2]#*=}

passwd=$($INI_CONFIG -f $DB_CONF -s Connection -k passwd -G)

# define function check_mysql_connection

check_mysql_connection(){
  local RET
  if [[ -z $host ]] ; then
    echo "mysql -D$database -u$login -p'$passwd'" >/tmp/proc.~$$~
    echo exit | mysql -D$database -u$login -p"$passwd" 2>>/tmp/proc.~$$~
  else
    echo "mysql -D$database -u$login -p'$passwd' -h$host" >/tmp/proc.~$$~
    echo exit | mysql -D$database -u$login -p"$passwd" -h$host 2>>/tmp/proc.~$$~
  fi
  return $?
}

# try to connect to database (give up after 3 failues)

RET=0
for i in 1 2 3 ; do
  check_mysql_connection
  RET=$?
  [[ $RET = 0 ]] && break
  sleep 5
done

[[ $RET != 0 ]] && cat /tmp/proc.~$$~
rm -f /tmp/proc.~$$~

exit $RET
