#!/bin/bash

[[ -w /etc/passwd ]] || {
  echo "${0##*/}: Only root can run this program, not ${LOGNAME:-$EUID}"
  exit 1
}

unset YES
while getopts y c ; do
  case $c in
    y) YES=1 ;;
  esac
done

EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
ETC_DEFAULT=$(readlink -f /etc/default/$EPAGES_PROJECT)
[[ -f $ETC_DEFAULT ]] && . $ETC_DEFAULT

[[ -n $EPAGES ]] && EPAGES_D=${EPAGES%/*} || EPAGES_D=/srv/epages
[[ -n $EPAGES_CONFIG ]] && DATABASE_D=$EPAGES_CONFIG/Database.d || DATABASE_D=$EPAGES_D/eproot/Shared/Config/Database.d

#
# ask_del
#

ask_del () {
  local YES=$1
  local WHAT=$2
  local PARTS=$(echo "$3" | sed 's,^,  ,')
  local ans

  echo -e "\nInstalled ePages $WHAT:" ; echo "$PARTS"
  [[ -n $YES ]] && return 0

  echo -en "\nDo you really want to delete those ePages $WHAT? (y/n): "
  read ans
  [[ $ans = y || $ans = Y ]] && return 0 || return 1
}

#
# stop epages
#

for i in $EPAGES_PROJECT mongod epagesj epages-solr epages-solr-slave ; do
  if grep -qi ubuntu /etc/os-release 2>/dev/null ; then
    PERL5LIB= /usr/sbin/sysv-rc-conf $i off >/dev/null 2>&1
  else
    PERL5LIB= /sbin/chkconfig --del $i >/dev/null 2>&1
  fi
  [[ $i = $EPAGES_PROJECT && -x /etc/init.d/$i ]] && /etc/init.d/$i stop_cron >/dev/null
  [[ -x /etc/init.d/$i ]] && /etc/init.d/$i stop
done

#
# umount/unshare
#

if [[ -x /var/epages/epages-fs.sh ]] ; then
  /var/epages/epages-fs.sh -unmount 2>/dev/null
  /var/epages/epages-fs.sh -unshare 2>/dev/null
fi

#
# delete RPMs
#

unset INSTALLED HAS_RPM
if type -p repoquery >/dev/null 2>&1 ; then
  INSTALLED=$(repoquery -a --qf "%{repoid} %{name}" | awk '/epages/{print $2}')
  HAS_RPM=yum
elif type -p yum >/dev/null 2>&1 ; then
  INSTALLED="$INSTALLED$'\n'"$(yum list installed | tr '\n' '"' | sed 's," ,,g' | tr '"' '\n' | awk '/epages/{sub(/\..*/,""); print $1}')
  HAS_RPM=yum
elif type -p zypper >/dev/null 2>&1 ; then
  INSTALLED=$(for i in $(cat /etc/zypp/repos.d/epages-*.repo \
              | awk -F\[ '/^[[\t ]]*\[epages.*[^0-9]\]/{print $2}') ; do
                zypper mr -e ${i%]}
                zypper search -i -r ${i%]}
                zypper mr -d ${i%]}
              done | awk '/^i \|/{print $3}')
  HAS_RPM=zypper
elif type -p dpkg >/dev/null 2>&1 ; then
  INSTALLED=$(dpkg --list | awk '/epages/{print $2}')
fi

if [[ -n $HAS_RPM ]] ; then
  INSTALLED="$INSTALLED$'\n'"$(rpm -qa --qf '%{NAME}\n' | grep epages)
  unset j
  for i in $(echo "$INSTALLED" | sort -r | uniq) ; do
    rpm --quiet -q $i && j="$j $i"
  done
  INSTALLED=$j
fi

if [[ -n $INSTALLED && -n $HAS_RPM ]] ; then
  if ask_del "$YES" RPMs "$(echo $INSTALLED | tr ' ' '\n')" ; then
    [[ -x $EPAGES/bin/use-local-httpd.sh ]] && $EPAGES/bin/use-local-httpd.sh -u

    unset RELEASE_INSTALLED
    for i in $INSTALLED ; do
      [[ $i = epages-release ]] && { RELEASE_INSTALLED=1; continue; }
      echo rpm --nodeps -v -e $i
      rpm --nodeps -v -e $i
    done
  fi
  [[ -n $RELEASE_INSTALLED ]] && rpm --nodeps -v -e epages-release
elif [[ -n $INSTALLED ]] ; then
  if ask_del "$YES" DEBs "$(echo $INSTALLED | tr ' ' '\n')" ; then
    [[ -x $EPAGES/bin/use-local-httpd.sh ]] && $EPAGES/bin/use-local-httpd.sh -u
    apt-get purge $INSTALLED
  fi
fi

#
# delete users
#

mkdir -p "$EPAGES_D"/etc/
for i in ep_appl ep_db ep_web epagesj eprunapp eprunweb epmysql epmysqlma ; do
  echo "user $i" >> "$EPAGES_D"/etc/created-accounts
done
for i in ep_web epages epagesj epmysql ; do
  echo "group $i" >> "$EPAGES_D"/etc/created-accounts
done

INSTALLED=$(sort -r "$EPAGES_D"/etc/created-accounts 2>/dev/null | uniq | while read type name ; do
  if [[ $type = user ]] ; then
    getent passwd $name >/dev/null 2>&1 && echo $type $name
  elif [[ $type = group ]] ; then
    getent group $name >/dev/null 2>&1 && echo $type $name
  fi
done)

if [[ -n $INSTALLED ]] ; then
  if ask_del "$YES" users/groups "$INSTALLED" ; then
    echo "$INSTALLED" | while read type name ; do
      if [[ $type = user ]] ; then
        echo userdel -r $name
        userdel -r $name 2>/dev/null
      elif getent group $name >/dev/null 2>&1 ; then
        echo groupdel $name
        groupdel $name 2>/dev/null
        grep -v "^$name:" /etc/group > /tmp/pid.~$$~ && cat /tmp/pid.~$$~ > /etc/group
        rm -f /tmp/pid.~$$~
      fi
    done
  fi
fi

#
# delete databases
#
mysqld_name() {
  local FPATH=$(systemctl --property=FragmentPath show mariadb 2>/dev/null)
  [[ -n ${FPATH#FragmentPath=} ]] && { echo mariadb; return; }

  [[ -f /etc/init.d/mysql ]] && { echo mysql; return; }
  [[ -f /etc/init.d/mysqld ]] && { echo mysqld; return; }
}

EPAGES_MYSQLD=$(mysqld_name)
if [[ -n $EPAGES_MYSQLD ]] && ls $DATABASE_D/*.conf >/dev/null 2>&1 ; then

  unset INSTALLED
  while read i ; do
    DATASOURCE=$(awk -F= "/^datasource=/{print \$2}" $i)
    DATASOURCE=${DATASOURCE// /}
    [[ $DATASOURCE = ${DATASOURCE#dbi:mysql:} ]] && continue
    DB=$(awk -F= "/^Database=/{print \$2}" $i)
    [[ -z ${DB// /} || ${DB// /} = mysql ]] && continue
    INSTALLED="$INSTALLED $DB"
  done < <(ls -d $DATABASE_D/*.conf)
  INSTALLED=$(echo $INSTALLED | tr ' ' '\n')

  if [[ -n $INSTALLED ]] ; then
    if ask_del "$YES" databases "$INSTALLED" ; then
      service $EPAGES_MYSQLD stop >/dev/null 2>&1

      PATH=$PATH:/usr/bin:/bin
      mysqld_safe --skip-grant-tables --skip-networking >/dev/null 2>&1 &

      COUNT=0
      while [[ $COUNT -lt 20 ]] ; do
        sleep 1
        echo exit | mysql 2>/dev/null && break
        COUNT=$(($COUNT + 1))
      done

      for i in $INSTALLED ; do
        echo "drop database $i;"
        echo "drop database $i;" | mysql >/dev/null 2>&1
        [[ -d /var/lib/mysql/$i ]] || continue
        echo "rm -rf /var/lib/mysql/$i"
        rm -rf /var/lib/mysql/$i
      done
      echo service $EPAGES_MYSQLD start
      pkill mysqld >/dev/null 2>&1
      sleep 2
      service $EPAGES_MYSQLD start >/dev/null 2>&1
    fi
  fi
fi

#
# delete files
#

MONGOD=/etc/init.d/mongod
grep -q epages $MONGOD 2>/dev/null || unset MONGOD

unset INSTALLED
MORE="'/etc/default/$EPAGES_PROJECT*' '/etc/init.d/$EPAGES_PROJECT*' '/etc/sysconfig/$EPAGES_PROJECT*' '/etc/*/rc?.d/*$EPAGES_PROJECT*'"
[[ $EPAGES_PROJECT = ${EPAGES_PROJECT#epages} ]] || unset MORE
for i in $EPAGES_D \
    /etc/default/epages* \
    /etc/sysconfig/epages* \
    /etc/init.d/epages* \
    '/etc/*/rc?.d/*epages*' \
    $MORE \
    '/etc/yum.repos.d/*epages*' \
    '/var/cache/yum/epages-*' \
    '/etc/zypp/repos.d/*epages*' \
    '/var/cache/zypp/*/epages-*' \
    /var/epages \
    /var/tmp/ini-config.sh \
    /var/tmp/epages-fs.sh $MONGOD
do
  eval ls -d1 $i >/dev/null 2>&1 && { [[ -z $INSTALLED ]] && INSTALLED="$i" || INSTALLED="$INSTALLED $i"; }
done
if ask_del "$YES" files "$(echo "$INSTALLED" | tr ' ' '\n')" ; then
  if [[ $HAS_RPM = yum ]] ; then
    echo yum clean all
    yum clean all
  elif [[ $HAS_RPM = zypper ]] ; then
    echo zypper clean -a
    zypper clean -a
    zypper refresh
  elif type -p apt-get >/dev/null 2>&1 ; then
    apt-get clean
    rm -f /var/lib/apt/lists/ep-server_apt_dists_stable_non-free_binary-*_Packages
  fi
  echo rm -rf $INSTALLED
  eval rm -rf $INSTALLED
fi

exit 0
