#!/bin/bash

#==============================================================================
# epages functions
#==============================================================================

fun_epages_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL EPAGES_ARCH local_timezone

[[ -n $fun_httpd_sourced ]] || . $(dirname $BASH_SOURCE)/fun-httpd.sh
export httpd_name httpd_confd

[[ -n $fun_mysql_sourced ]] || . $(dirname $BASH_SOURCE)/fun-mysql.sh
export mysqld_name mysql_installed mysqld_installed

[[ -n $fun_process_sourced ]] || . $(dirname $BASH_SOURCE)/fun-process.sh
export get_pids

[[ -n $fun_term_sourced ]] || . $(dirname $BASH_SOURCE)/fun-term.sh
export Y COLUMNS

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file repair_name repair_number chkconfig_do

[[ -n $fun_account_sourced ]] || . $(dirname $BASH_SOURCE)/fun-account.sh
export get_user_info get_group_info

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export get_host_info get_ip_info guess_addr_type my_ips
export get_host_ip get_host_name resolve_host_info

[[ -n $fun_java_sourced ]] || . $(dirname $BASH_SOURCE)/fun-java.sh
export mongod_name test_version

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export etc_default get_cartridge_version get_cartridge_fullversion
export internal_version component_installed epages_status
export epages_service run_cron EPAGES_CRON_NO_SCRIPT EPAGES_CRON_IGNORED
export update_epages_environment link_etc create_etc_default link_epages
export enable_ipv6 set_epages_conf set_local_ips
export start_epages_if_stopped start_epagesj_if_stopped

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# etc_default
#..............................................................................
etc_default() {
  local USAGE='
syntax: etc_default
description: returns path to epages default file (e.g. /etc/default/epages6)
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local PREFIX="$1"
  local PREFIXED=$PREFIX/etc/default/epages6
  local EPT PROJECT_FILE

  [[ -n $PREFIX && -f $PREFIXED ]] && { echo $PREFIXED; return; }
  [[ -n $PREFIX && -f $PREFIXED.ept ]] && EPT=$PREFIXED

  EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
  if [[ -n $EPAGES_PROJECT ]] ; then
    PROJECT_FILE=/etc/default/$EPAGES_PROJECT
    [[ -f $PROJECT_FILE ]] && { echo $PROJECT_FILE; return; }
    [[ -f $PROJECT_FILE.ept ]] && EPT=${EPT:-$PROJECT_FILE}
  fi

  if [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
    PREFIX=$(rpm -q epages-common --qf '[%{INSTPREFIXES}\n]' 2>/dev/null | head -1)
  else
    PREFIX=
  fi
  PREFIXED=$PREFIX/etc/default/epages6
  [[ -n $PREFIX && -f $PREFIXED ]] && { echo $PREFIXED; return; }
  [[ -n $PREFIX && -f $PREFIXED.ept ]] && EPT=${EPT:-$PREFIXED}

  [[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
  if [[ -n $EPAGES_PROJECT ]] ; then
    PROJECT_FILE=${PROJECT_FILE:-/etc/default/$EPAGES_PROJECT}
    [[ -f $PROJECT_FILE ]] && { echo $PROJECT_FILE; return; }
    [[ -f $PROJECT_FILE.ept ]] && EPT=${EPT:-$PROJECT_FILE}
  fi

  [[ -f /etc/default/epages6 ]] && { echo /etc/default/epages6; return; }

  local DEFAULT_FILE=$(ls /etc/default/epages6* 2>/dev/null)
  [[ $(echo "$DEFAULT_FILE" | egrep -v '(\.ept|~)$' | egrep . | wc -l) -eq 1 ]] \
  && { echo "$DEFAULT_FILE" | egrep -v '(\.ept|~)$'; return; }
  [[ $(echo "$DEFAULT_FILE" | egrep '\.ept$' | wc -l) -eq 1 ]] \
  && { echo "$DEFAULT_FILE" | egrep '\.ept$' | sed 's,\.ept$,,'; return; }

  echo ${EPT:-${PROJECT_FILE:-/etc/default/epages6}}
}

[[ -f $(readlink -f $(etc_default)) ]] && . $(etc_default)
[[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6

#..............................................................................
# get_business_units
#..............................................................................
get_business_units() {
  local USAGE='
syntax: get_business_units [CARTRIDGE]
description: returns list of business units
  CARTRIDGE - with CARTRIDGE (default: all business units)
example: get_business_units DE_EPAGES::RemoteSearch
'
  [[ $1 != ${1#-} || $# -gt 1 ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -n $EPAGES_CONFIG ]] || return

  local CARTRIDGE=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local BU_CONF CARTRIDGES_CONF STORETYPE DATABASE DATASOURCE

  for BU_CONF in $(ls -d "$EPAGES_CONFIG"/Database.d/*.conf 2>/dev/null) ; do
    CARTRIDGES_CONF="$EPAGES_CONFIG"/Cartridges.d/${BU_CONF##*/}
    # BU must have Cartridges.d/*.conf
    [[ -f $CARTRIDGES_CONF ]] || continue
    # BU must have required cartridge
    if [[ -n $CARTRIDGE ]] ; then
      $INI_CONFIG -f $CARTRIDGES_CONF -s Cartridges -k $CARTRIDGE -G | grep -q .
      [[ $? = 0 ]] || continue
    fi
    # BU must have datasource dbi:mysql in Database.d/*.conf
    DATASOURCE=$($INI_CONFIG -f $BU_CONF -s Connection -k datasource -G)
    [[ $DATASOURCE =~ ^dbi:mysql ]] || continue
    # BU must have StoreType in Database.d/*.conf but not epagesj/ASPoolDB
    STORETYPE=$($INI_CONFIG -f $BU_CONF -s Connection -k StoreType -G)
    [[ -z $STORETYPE || $STORETYPE = epagesj || $STORETYPE = ASPoolDB ]] && continue
    # BU must have Database in Database.d/*.conf but not epagesj/mysql
    DATABASE=$($INI_CONFIG -f $BU_CONF -s Connection -k Database -G)
    [[ -z $DATABASE || $DATABASE = epagesj || $DATABASE = mysql ]] && continue
    # BU found
    BU_CONF=${BU_CONF##*/}
    echo -n "${BU_CONF%.conf} "
  done
}

#..............................................................................
# get_cartridge_fullversion
#..............................................................................
get_cartridge_fullversion() {
  local USAGE='
syntax: get_cartridge_version CARTRIDGE
description: get full version (as 6.9.1.1) of CARTRIDGE in DE_EPAGES/ if CARTRIDGE !~ ,::,
             get full version (as 6.9.1.1) of CARTRIDGE in Cartridges/ if CARTRIDGE =~ ,::,
example: get_cartridge_version Core
example: get_cartridge_version Training::GuestBook
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -n $EPAGES_CONFIG ]] || return

  local CONFFILES
  if grep -q '^\[Cartridges\]' "$EPAGES_CONFIG"/Database.d/*.conf >/dev/null 2>&1 ; then
    CONFFILES="$EPAGES_CONFIG"/Database.d/*.conf
  elif ls "$EPAGES_CONFIG"/Cartridges.d/*.conf >/dev/null 2>&1 ; then
    CONFFILES="$EPAGES_CONFIG"/Cartridges.d/*.conf
  elif [[ -f "$EPAGES_CONFIG"/Cartridges.conf ]] ; then
    CONFFILES="$EPAGES_CONFIG"/Cartridges.conf
  fi

  [[ -n $CONFFILES ]] || return
  local CARTRIDGE="$1"
  [[ $CARTRIDGE = ${CARTRIDGE#*::} ]] && CARTRIDGE="DE_EPAGES::$CARTRIDGE"
  awk -F= "/^$CARTRIDGE=[0-9.]+/{print \$2;exit 0}" $CONFFILES
}

#..............................................................................
# get_cartridge_version
#..............................................................................
get_cartridge_version() {
  local USAGE='
syntax: get_cartridge_version CARTRIDGE
description: get 3-part version (as 6.9.1) of CARTRIDGE in DE_EPAGES/ if CARTRIDGE !~ ,::,
             get 3-part version (as 6.9.1) of CARTRIDGE in Cartridges/ if CARTRIDGE =~ ,::,
example: get_cartridge_version Core
example: get_cartridge_version Training::GuestBook
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  local VRS=$(get_cartridge_fullversion $1)
  local CNT=0
  for part in ${VRS//./ } ; do
    CNT=$((CNT + 1))
    [[ $CNT = 4 ]] && break
    [[ $CNT != 1 ]] && echo -n .
    echo -n $part
  done
}

#..............................................................................
# internal_version
#..............................................................................
internal_version() {
  local USAGE='
syntax: internal_version VERSION
description: return internal version (number) for dotted VERSION
example: internal_version 6.16.1 -> 6016001
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  printf "%d%03d%03d" ${1//./ }
}

#..............................................................................
# component_installed
#..............................................................................
component_installed() {
  local USAGE='
syntax: component_installed PKG
returns: 1 - if PKG is installed, 0 - else
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local pkg
  case $1 in
    appsrv) [[ -x ${EPSERVICE:-/srv/epages/eproot/Service/epagesService} ]]
      return $? ;;
    common) [[ -f ${EPAGES_CARTRIDGES:-/srv/epages/eproot/Cartridges}/DE_EPAGES/Makefile.PL ]]
      return $? ;;
    j) [[ -d ${EPAGESJ:-/srv/epages/eproot/j} ]]
      return $? ;;
    mysql) mysql_installed || mysqld_installed
      return $? ;;
    mysqld) mysqld_installed
      return $? ;;
    nodejs) [[ -x ${EPAGES_PERL:-/srv/epages/eproot/Perl}/bin/nodejs.sh ]]
      return $? ;;
    perl) [[ -x ${PERL:-/srv/epages/eproot/Perl/bin/perl} ]]
      return $? ;;
    reqsrv) [[ -x ${EP_RR:-/srv/epages/eproot/RequestRouter/RequestRouter} ]]
      return $? ;;
    ascsrv) [[ -x ${EP_ASC:-/srv/epages/eproot/ASPoolDBCacheServer/ASPoolDBCacheServer} ]]
      return $? ;;
    websrv) [[ -x ${EPAGES_APACHE:-/srv/epages/eproot/Apache}/bin/httpd ]]
      return $? ;;
    webconf) [[ -n $(httpd_name) && -f $(httpd_confd)/zzz-epages-httpd.conf ]]
      return $? ;;
    patch*|requires) return 0 ;;
  esac
}

#..............................................................................
# epages_status
#..............................................................................
epages_status() {
  local USAGE='
syntax: epages_status [-v] SERVICE
arguments:
  -v      - verbose output
  SERVICE - one of: AS WS RR DB ASC
returns:
  0 - running
  1 - stopped
  2 - missing/unknown/error
'
  local VERBOSE
  [[ $1 = -v ]] && { VERBOSE=1; shift; }
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 2; }

  local service=$1
  local os_service STATUS
  case $service in
    DB) os_service=$(mysqld_name) ; component_installed mysqld || STATUS=2 ;;
    WS) component_installed websrv || { os_service=$(httpd_name) ; component_installed webconf || STATUS=2; } ;;
    AS) component_installed appsrv || { os_service=epagesService ; STATUS=2; } ;;
    RR) component_installed reqsrv || { os_service=RequestRouter ; STATUS=2; } ;;
    ASC) component_installed ascsrv || { os_service=ASPoolDBCacheServer ; STATUS=2; } ;;
    *)  echo "$USAGE" 1>&2; return 2 ;;
  esac
  if [[ $STATUS = 2 ]] ; then
    [[ -n $VERBOSE ]] && echo "$os_service service missing"
    return 2
  fi

  # operating system run scripts

  if [[ -n $os_service ]] ; then
    local FPATH=$(systemctl --property=FragmentPath show $os_service 2>/dev/null)
    if [[ -z ${FPATH#FragmentPath=} ]] ; then
      # we'll use sysvinit, not systemd
      [[ -x /etc/init.d/$os_service ]] || STATUS=4
    fi

    if [[ -n $VERBOSE ]] ; then
      service $os_service status ; STATUS=${STATUS:-$?}
    else
      service $os_service status >/dev/null 2>&1 ; STATUS=${STATUS:-$?}
    fi
    case $STATUS in
      0) return 0 ;;
      [123]) return 1 ;;
      *) return 2 ;;
    esac
  fi

  # epages run scripts

  local user cmd pids
  case $service in
    AS) user=${EPAGES_APPUSER:-eprunapp}
        cmd=${EPSERVICE:-/srv/epages/eproot/Service/epagesService} ;;
    RR) user=${EPAGES_APPUSER:-eprunapp}
        cmd=${EP_RR:-/srv/epages/eproot/RequestRouter/RequestRouter} ;;
    ASC) user=${EPAGES_APPUSER:-eprunapp}
        cmd=${EP_ASC:-/srv/epages/eproot/ASPoolDBCacheServer/ASPoolDBCacheServer} ;;
    WS) user=${EPAGES_WEBUSER:-eprunweb}
        cmd=${EPAGES_APACHE:-/srv/epages/eproot/Apache}/bin/httpd ;;
  esac

  pids=$(get_pids -u $user -c "$cmd") || STATUS=2
  [[ -n $pids ]] && STATUS=0 || STATUS=1

  if [[ -n $VERBOSE ]] ; then
    case $STATUS in
      0) echo "$(basename $cmd) (pid $pids) is running..." ;;
      1) echo "$(basename $cmd) is stopped" ;;
      2) echo "$(basename $cmd) status unknown" ;;
    esac
  fi
  return $STATUS
}

#..............................................................................
# start_epages_if_stopped
#..............................................................................
start_epages_if_stopped() {
  local USAGE='
syntax: start_epages_if_stopped [-v] [SERVICE...]
description: start epages SERVICE if stopped; ignore epages cronjobs
arguments:
  -v      - verbose output
  SERVICE - one/more of: AS WS RR DB ASC (any service if missing)
returns: success or failure
'
  local VERBOSE service
  [[ $1 = -v ]] && { VERBOSE=$1; shift; }
  [[ -n $1 && $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

  for service in ${@:-WS DB ASC RR AS} ; do
    epages_status $VERBOSE $service
    # do nothing unless stopped
    [[ $? = 1 ]] || continue
    case $service in
      WS)  EPAGES_IGNORE=cron /etc/init.d/$EPAGES_PROJECT start_httpd || return $? ;;
      DB)  EPAGES_IGNORE=cron /etc/init.d/$EPAGES_PROJECT start_db || return $? ;;
      ASC) EPAGES_IGNORE=cron /etc/init.d/$EPAGES_PROJECT start_asc || return $? ;;
      RR)  EPAGES_IGNORE=cron /etc/init.d/$EPAGES_PROJECT start_rr || return $? ;;
      AS)  EPAGES_IGNORE=cron /etc/init.d/$EPAGES_PROJECT start_service || return $? ;;
      *)  echo "$USAGE" 1>&2; return 1 ;;
    esac
  done
  return 0
}

#..............................................................................
# start_epagesj_if_stopped
#..............................................................................
start_epagesj_if_stopped() {
  local USAGE='
syntax: start_epagesj_if_stopped [-v]
description: start epages java/mongod services if stopped
arguments:
  -v      - verbose output
returns: success or failure
'

  local VERBOSE service
  [[ $1 = -v ]] && { VERBOSE=$1; shift; }
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # do nothing if no epagesj service installed
  grep -ql '^[^#]*\bidentifier=epages.daemon=' /etc/init.d/* || return 0

  local MONGOD_NAME=$(mongod_name)
  # if service $MONGOD_NAME status returns success then it is already started
  if [[ -n $MONGOD_NAME && -n $VERBOSE ]] && ! service $MONGOD_NAME status ; then
    service $MONGOD_NAME start || return 1
  elif [[ -n $MONGOD_NAME ]] && ! service $MONGOD_NAME status >/dev/null 2>&1 ; then
    service $MONGOD_NAME start || return 1
  fi

  grep -l '^[^#]*\bidentifier=epages.daemon=' /etc/init.d/* | while read service ; do
    # if $service status returns success then it is already started
    if [[ -n $VERBOSE ]] ; then
      $service status && continue
    else
      $service status >/dev/null 2>&1 && continue
    fi
    $service start || return 1
  done

  return 0
}

#..............................................................................
# epages_service
#..............................................................................
epages_service() {
  local USAGE='
syntax: epages_service {stop|start|restart|status} SERVICE...
        does nothing if run script (/etc/init.d/epages6) not exist
arguments:
  stop - stops SERVICE...
  start - starts SERVICE... unless running
  restart - (re)starts SERVICE...
  status - shows status of SERVICE...
  SERVICE - list of: AS WS RR ASC DB
'
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local RUNSCRIPT="$RPM_INSTALL_PREFIX"/etc/init.d/epages6
  [[ -x $RUNSCRIPT ]] || RUNSCRIPT=/etc/init.d/$(basename $(etc_default))
  [[ -x $RUNSCRIPT ]] || return 1

  local TASK=$1 ; shift
  local SERVICE SERVICE_NAME STATUS

  declare -A SERVICE_NAME
  SERVICE_NAME[AS]=service
  SERVICE_NAME[WS]=httpd
  SERVICE_NAME[RR]=rr
  SERVICE_NAME[ASC]=asc
  SERVICE_NAME[DB]=db

  local VERBOSE
  [[ $TASK = status ]] && VERBOSE=-v
  for SERVICE in $@ ; do
    [[ -z ${SERVICE_NAME[$SERVICE]} ]] && continue
    epages_status $VERBOSE $SERVICE ; STATUS=$?
    [[ $STATUS = 2 ]] && continue
    case $TASK in
      status) continue ;;
      *start) [[ $TASK = start && $STATUS = 0 ]] && continue
        IGNORE_INI=1 $RUNSCRIPT start_${SERVICE_NAME[$SERVICE]} >/dev/null 2>&1 ;;
      stop) [[ $STATUS != 0 ]] && continue
        $RUNSCRIPT stop_${SERVICE_NAME[$SERVICE]} >/dev/null 2>&1 ;;
    esac
  done
}

#..............................................................................
# run_cron
#..............................................................................
EPAGES_CRON_NO_SCRIPT=99
EPAGES_CRON_IGNORED=98
run_cron() {
  local USAGE='
syntax: run_cron {start|stop|show} [USER]
arguments:
  USER: starts/stops/shows epages cron processes (of USER)
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local RUN=$1
  local user=$2

  local sched="${EPAGES:-/srv/epages/eproot}"/bin/epagesScheduler.sh
  local crond=cron
  [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%r} ]] && crond=crond

  test -x "$sched" || return $EPAGES_CRON_NO_SCRIPT
  [[ -n $EPAGES_IGNORE_CRON ]] && return $EPAGES_CRON_IGNORED

  local TASK=" ${user:-epages} cron jobs..."
  case "$RUN" in
    start) TASK=$(Y -i Starting $TASK) ;;
    stop)  TASK=$(Y -i Stopping $TASK) ;;
    show)  TASK=$(Y -i Showing $TASK) ;;
    *) echo "$USAGE" 1>&2; return 1 ;;
  esac
  echo $TASK

  service $crond status >/dev/null 2>&1 || { service $crond status ; return $?; }

  if [[ -n $user ]] ; then
    local SU_EXE=runuser
    $SU_EXE -c true 2>/dev/null || SU_EXE=su
    $SU_EXE $user -c ". \"$(etc_default)\" ; LOGNAME=$user \"$sched\" $RUN" \
    2>/dev/null | grep -v '^S[a-z]*ing '
  else
   "$sched" $RUN | grep -v '^S[a-z]*ing '
  fi

  service $crond status
  return $?
}

#..............................................................................
# update_epages_environment
#..............................................................................
update_epages_environment() {
  local USAGE='
syntax: update_epages_environment [-i INSTANCE] [EPAGES*...]
options:
  -i - if set to 1 (first install), all variables are set
       if set to 0, nothing happens
       if set to 2 (or higher), some varibles are not set (marked with *)
description: equalizes variables between /etc/default/epages6 (as defd.
  on command line) and /etc/sysconfig/epages6 . w/o arguments, following
  variables are equalized:

  /etc/default/epages6                    /etc/sysconfig/epages6
  ----------------------------------------------------------------------
  EPAGES                                  RPM_INSTALL_PREFIX/eproot
  EPAGES_PROJECT                          EPAGES_INSTALL_RUN_SCRIPT
  EPAGES_TZ*                              -
  EPAGES_IGNORE_DEMOSHOP*                 EPAGES_INSTALL_IGNORE_DEMOSHOP
  EPAGES_BUSINESS_UNITS*                  EPAGES_INSTALL_BUSINESS_UNITS
  EPAGES_IGNORE_SITE/EPAGES_IGNORE_STORE* EPAGES_INSTALL_BUSINESS_UNITS
  HTTPD_OPTS*                             EPAGES_INSTALL_HTTPD_OPTS
  EPAGES_APPUSER                          EPAGES_INSTALL_AS_RUNUSER
  EPAGES_WEBUSER                          EPAGES_INSTALL_WS_RUNUSER
  EPAGES_WEBGROUP                         EPAGES_INSTALL_WS_GROUP:*GID
  EPAGESJ_USER                            EPAGES_INSTALL_EPAGESJ_USER
  EPAGESJ_HOST                            EPAGES_INSTALL_EPAGESJ_HOST
  EPAGESJ_PORT                            EPAGES_INSTALL_EPAGESJ_PORT

example 1: equalize EPAGES_INSTALL_RUN_SCRIPT and EPAGES_PROJECT:
  update_epages_environment EPAGES_PROJECT
example 2: equalize all default variables as defd above:
  update_epages_environment -i 1
'
  [[ $1 != ${1#-} && $1 != -i ]] && { echo "$USAGE" 1>&2; return 1; }
  local INSTANCE=2
  [[ $1 = -i ]] && { shift ; INSTANCE=$1 ; shift; }
  [[ $INSTANCE = 0 ]] && return 0

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  local ETC_INSTALL=/etc/sysconfig/epages6

  backup_file "$ETC_DEFAULT".ept || return 1
  [[ -f $(readlink -f "$ETC_DEFAULT") ]] && . "$ETC_DEFAULT"
  [[ -f $ETC_INSTALL ]] && . $ETC_INSTALL

  local VARIABLES="EPAGES EPAGES_PROJECT EPAGES_TZ EPAGES_IGNORE_DEMOSHOP
                   EPAGES_BUSINESS_UNITS HTTPD_OPTS EPAGES_APPUSER EPAGES_WEBUSER
                   EPAGES_WEBGROUP EPAGESJ_USER EPAGESJ_HOST EPAGESJ_PORT"
  [[ $# != 0 ]] && VARIABLES=$@

  local default_var install_var val
  for default_var in $VARIABLES ; do
    unset val install_var
    case $default_var in
      EPAGES_PROJECT)
        val=$(repair_name ${EPAGES_PROJECT:-${EPAGES_INSTALL_RUN_SCRIPT:-epages6}})
        install_var=EPAGES_INSTALL_RUN_SCRIPT ;;
      EPAGES)
        val=${EPAGES:-${RPM_INSTALL_PREFIX:-/srv/epages}/eproot}
        install_var= ;;
      EPAGES_TZ) [[ $INSTANCE != 1 ]] && continue
        val=${EPAGES_TZ:-$(local_timezone)}
        install_var= ;;
      HTTPD_OPTS) [[ $INSTANCE != 1 ]] && continue
        val=${EPAGES_INSTALL_HTTPD_OPTS:--DSSL}
        [[ $val = ${val#*-DSSL} ]] && val="$val -DSSL"
        install_var=EPAGES_INSTALL_HTTPD_OPTS ;;
      EPAGES_IGNORE_DEMOSHOP) [[ $INSTANCE != 1 ]] && continue
        val=$EPAGES_INSTALL_IGNORE_DEMOSHOP
        install_var=EPAGES_INSTALL_IGNORE_DEMOSHOP ;;
      EPAGES_BUSINESS_UNITS) [[ $INSTANCE != 1 ]] && continue
        val=$EPAGES_INSTALL_BUSINESS_UNITS
        [[ $val = ${val#*Site} ]] && EPAGES_IGNORE_SITE=1 || EPAGES_IGNORE_SITE=
        $INI_CONFIG -f $ETC_DEFAULT -k EPAGES_IGNORE_SITE -v "$EPAGES_IGNORE_SITE" -i
        [[ $val = ${val#*Store} ]] && EPAGES_IGNORE_STORE=1 || EPAGES_IGNORE_STORE=
        $INI_CONFIG -f $ETC_DEFAULT -k EPAGES_IGNORE_STORE -v "$EPAGES_IGNORE_STORE" -i
        install_var=EPAGES_INSTALL_BUSINESS_UNITS ;;
      EPAGES_APPUSER)
        val=$(get_user_info AS_RUN) ; val=${val%:*}
        install_var=EPAGES_INSTALL_AS_RUNUSER ;;
      EPAGES_WEBUSER)
        val=$(get_user_info WS_RUN) ; val=${val%:*}
        install_var=EPAGES_INSTALL_WS_RUNUSER ;;
      EPAGES_WEBGROUP)
        val=$(get_group_info WS_RUN)
        $INI_CONFIG -f $ETC_INSTALL -k EPAGES_INSTALL_WS_GROUP -v "${val%:*}" -i
        install_var= ;;
      EPAGESJ_USER)
        val=$(get_user_info J) ; val=${val%:*}
        install_var=EPAGES_INSTALL_EPAGESJ_USER ;;
      EPAGESJ_HOST)
        val=$(get_host_name J)
        $INI_CONFIG -f $ETC_INSTALL -k EPAGES_INSTALL_$default_var -v "$val" -i
        [[ $(guess_addr_type "$val") = IPv6 ]] && val="[$val]" ;;
      EPAGESJ_PORT)
        val=${EPAGES_INSTALL_EPAGESJ_PORT:-$EPAGESJ_PORT}
        [[ -z $val ]] && default_var= || install_var=EPAGES_INSTALL_EPAGESJ_PORT ;;
      *) continue ;;
    esac
    if [[ -n $default_var ]] ; then
      eval $default_var=\"$val\"
      $INI_CONFIG -f $ETC_DEFAULT -k $default_var -v "\"$val\"" -i
    fi
    if [[ -n $install_var ]] ; then
      $INI_CONFIG -f $ETC_INSTALL -k $install_var -v "\"$val\"" -i
    fi
  done

  return 0
}

#..............................................................................
# link_epages
#..............................................................................
link_epages () {
  local USAGE='
syntax: link_epages PREFIX
description: links /srv/epages to PREFIX (unless exists and or is equal)
'
  [[ -z $1 ]] && return
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local PREFIX=$(readlink -f $1)
  [[ $PREFIX = $(readlink -f /srv/epages) ]] && return
  [[ -d /srv/epages/eproot/bin ]] && return

  local DATE=.~$(date +%s)~

  rm -f /srv/epages/{docs,etc,eproot} /srv/epages 2>/dev/null
  for i in $(find /srv/epages/{*,.[^.]*} -maxdepth 0 -printf "%f " 2>/dev/null) ; do
    [[ -f $(readlink -f "$PREFIX"/$i) ]] || rm -f "$PREFIX"/$i 2>/dev/null
    [[ -e "$PREFIX"/$i ]] && mv /srv/epages/$i "$PREFIX"/$i$DATE
    [[ -e "$PREFIX"/$i ]] || mv /srv/epages/$i "$PREFIX"/
  done
  rmdir /srv/epages 2>/dev/null
  rm -f /srv/epages 2>/dev/null
  mv /srv/epages /srv/epages$DATE 2>/dev/null
  mkdir -m 755 /srv
  ( cd /srv ; ln -s "$PREFIX" epages )
}
  
#..............................................................................
# link_etc
#..............................................................................
link_etc () {
  local USAGE='
syntax: link_etc [default|init.d]
description: links /srv/epages/etc/{default,init.d}/epages6 to /etc/{default,init.d}
  (unless exists) and adds service epages6
options:
  if default or init.d is set, change just those files
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local CHANGE_RS=init.d
  local CHANGE_DEFAULT=default
  [[ $1 = default ]] && CHANGE_RS= || { [[ $1 = init.d ]] && CHANGE_DEFAULT=; }

  local ETC_DIR
  [[ -n $EPAGES ]] && ETC_DIR=$(dirname $EPAGES) || ETC_DIR=${RPM_INSTALL_PREFIX:-/srv/epages}
  ETC_DIR=$ETC_DIR/etc
  EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

  if [[ -n $CHANGE_RS ]] ; then
    backup_file "$ETC_DIR"/init.d/epages6.ept
    sed -i "s,_EPAGES_PROJECT_,$EPAGES_PROJECT,g" "$ETC_DIR"/init.d/epages6
    chown root "$ETC_DIR"/init.d/epages6
  fi

  for i in $CHANGE_RS $CHANGE_DEFAULT ; do
    [[ $i = $CHANGE_RS && -f $(readlink -f /etc/$i/$EPAGES_PROJECT) ]] && continue
    pushd /etc/$i >/dev/null || continue
    if [[ -f $(readlink -f "$ETC_DIR"/$i/epages6) ]] ; then
      rm -f $EPAGES_PROJECT
      ln -s "$ETC_DIR"/$i/epages6 $EPAGES_PROJECT
    fi
    popd >/dev/null
  done
  [[ -n $CHANGE_DEFAULT ]] && chmod 755 /etc/default/$EPAGES_PROJECT

  if [[ -n $CHANGE_RS ]] ; then
    chmod 750 /etc/init.d/$EPAGES_PROJECT
    chkconfig_do --del $EPAGES_PROJECT >/dev/null 2>&1
    chkconfig_do --add $EPAGES_PROJECT >/dev/null 2>&1
  fi
}

#..............................................................................
# create_etc_default
#..............................................................................
create_etc_default () {
  local USAGE='
syntax: create_etc_default FILE
description: creates (unless exists) /srv/epages/etc/default/epages6
  from /srv/epages/etc/default/FILE
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILE=$1
  local ETC_DIR
  [[ -n $EPAGES ]] && ETC_DIR=$(dirname $EPAGES) || ETC_DIR=${RPM_INSTALL_PREFIX:-/srv/epages}
  ETC_DIR=$ETC_DIR/etc
  EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

  [[ $EPAGES_PROJECT = $FILE ]] && return
  [[ -f $ETC_DIR/default/$EPAGES_PROJECT ]] && return
  ls $ETC_DIR/default/$FILE >/dev/null || return 1

  cp -p $ETC_DIR/default/$FILE $ETC_DIR/default/$EPAGES_PROJECT
  chmod 755 $ETC_DIR/default/$EPAGES_PROJECT
}

#..............................................................................
# set_local_ips
#..............................................................................
set_local_ips () {
  local USAGE='
syntax: set_local_ips {AS|RR}
description: sets LOCAL_IPS in /etc/default/epages6 (if distributed installation)
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SERVER=$1
  local IP=$(get_ip_info $SERVER)
  IP=${IP:-$(get_host_info $SERVER)}
  [[ $(guess_addr_type "$IP") = hostname ]] && IP=$(get_ip_by_name $IP)
  IP=${IP%% *}

  [[ -z $IP ]] && return 0

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  [[ -f "$ETC_DEFAULT" ]] || return

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local LOCAL_IPS=$($INI_CONFIG -G -k "LOCAL_IPS" -f "$ETC_DEFAULT")
  LOCAL_IPS=$(echo ${LOCAL_IPS#*-i})
  LOCAL_IPS=${LOCAL_IPS:+,$LOCAL_IPS,}
  LOCAL_IPS=${LOCAL_IPS:-,}
  [[ $LOCAL_IPS =~ ,$IP, ]] || LOCAL_IPS=$LOCAL_IPS$IP,

  LOCAL_IPS=$(echo ${LOCAL_IPS%,} | sed -r 's/,,+/,/g')
  LOCAL_IPS=${LOCAL_IPS#,}
  local IP_BINDING=-b
  [[ -n $LOCAL_IPS ]] && LOCAL_IPS="\"-i $LOCAL_IPS\"" || IP_BINDING=

  # write new $ETC_DEFAULT
  sed "s/^\(LOCAL_IPS\)=.*/\1=$LOCAL_IPS/;s/^\(IP_BINDING\)=.*/\1=$IP_BINDING/" \
  "$ETC_DEFAULT" > "$ETC_DEFAULT".new

  # backup/update existing $ETC_DEFAULT (if has changed)
  if ! cmp "$ETC_DEFAULT" "$ETC_DEFAULT".new >/dev/null 2>&1 ; then
    cp -p "$ETC_DEFAULT" "$ETC_DEFAULT".~$(date +%s)~
    cat "$ETC_DEFAULT".new > "$ETC_DEFAULT"
  fi
  rm -f "$ETC_DEFAULT".new
}

#..............................................................................
# enable_ipv6
#..............................................................................
enable_ipv6 () {
  local USAGE='
syntax: enable_ipv6 [CHECK_MY_IPS]
description: sets EPAGES_ENABLE_IPv6=1 if:
  - WebInterface.conf has IPv6 addresses
  - or if one of AS/RR/WS/J/MYSQL server variables use IPv6
  - or if CHECK_MY_IPS is used and my_ips have IPv6
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF="$EPAGES_CONFIG"/WebInterface.conf
  [[ -f "$WI_CONF" ]] || return

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local SET_IPv6 SECTION
  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  [[ -f $(readlink -f "$ETC_DEFAULT") ]] || return

  # if WebInterface.conf has IPv6 addresses then set EPAGES_ENABLE_IPv6=1
  if [[ -f "$EPAGES_CONFIG"/WebInterface.conf ]] ; then
    if grep -Pq '^DEDICATED=((.*:){7}|.*::[^.]*$)' "$EPAGES_CONFIG"/WebInterface.conf ; then
      # find DEDICATED= key with IPv6 address
      SET_IPv6=1
    else
      for SECTION in $($INI_CONFIG -f "$EPAGES_CONFIG"/WebInterface.conf -g) ; do
        # find IPv6 section
        [[ $SECTION = ${SECTION#*:} ]] && continue
        # skip IPv6 section with '*'
        [[ $SECTION = ${SECTION#*\*} ]] || continue
        # find section keys with port numbers
        $INI_CONFIG -f "$EPAGES_CONFIG"/WebInterface.conf -s "$SECTION" -g \
        | grep -q '=[0-9]' && SET_IPv6=1
      done
    fi
  fi

  if [[ -z $SET_IPv6 ]] ; then
    # if server use IPv6 then set EPAGES_ENABLE_IPv6=1
    for SERVER in AS RR; do
      if ADDR=$(get_ip_info $SERVER) && [[ $(guess_addr_type "$ADDR") = IPv6 ]] ; then
        SET_IPv6=1 ; break
      elif ADDR=$(get_host_info $SERVER) && [[ $(guess_addr_type "$ADDR") = IPv6 ]] ; then
        SET_IPv6=1 ; break
      fi
    done
  fi

  if [[ -n $SET_IPv6 ]] ; then
    $INI_CONFIG -f "$ETC_DEFAULT" -k EPAGES_ENABLE_IPv6 -v 1 -i 2>/dev/null
  fi

  # if my_ips have IPv6 then set EPAGES_ENABLE_IPv6=1
  [[ $1 = CHECK_MY_IPS ]] || return
  for ADDR in $(my_ips) ; do
    if [[ $(guess_addr_type "$ADDR") = IPv6 ]] ; then
      $INI_CONFIG -f "$ETC_DEFAULT" -k EPAGES_ENABLE_IPv6 -v 1 -i 2>/dev/null
      return
    fi
  done
}

#..............................................................................
# set_epages_conf
#..............................................................................
set_epages_conf () {
  local USAGE='
syntax: set_epages_conf [HOST [PORT]]
description: creates epages.conf from epages.conf.ept (unless exists)
             sets SystemDomainName and DefaultWebServiceProxy in epages.conf
'
  [[ $# -gt 2 ]] && { echo "$USAGE" 1>&2; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  . /etc/sysconfig/epages6

  local EPAGES_CONF="$EPAGES_CONFIG"/epages.conf
  [[ -f "$EPAGES_CONF" ]] && return

  backup_file "$EPAGES_CONF".ept

  local HOST_NAME=${1:-$(resolve_host_info WS)}
  HOST_NAME=${HOST_NAME%% *}
  local WS_HOST=$HOST_NAME
  [[ $WS_HOST != ${WS_HOST/:/} ]] && WS_HOST="[$WS_HOST]"
  local WS_PORT=${2:-$(repair_number "$EPAGES_INSTALL_WS_PORT" 80)}
  [[ $WS_PORT != 80 ]] && WS_HOST="$WS_HOST:$WS_PORT"

  $INI_CONFIG -f "$EPAGES_CONF" -s "DE_EPAGES::Object" -k SystemDomainName -v "$HOST_NAME" -i 2>/dev/null
  $INI_CONFIG -f "$EPAGES_CONF" -s "DE_EPAGES::Store" -k DefaultWebServiceProxy -r ':\/\/[^\/]+\/' -v "://$WS_HOST/" -i 2>/dev/null
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
