#!/bin/bash

#==============================================================================
# patch
#==============================================================================

fun_patch_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export check_prerequistes patch_epages patch_repo

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# check_prerequistes
#..............................................................................
check_prerequistes() {
  local USAGE='
syntax: check_prerequistes VERSION
description: checks if patch VERSION can be installed
'
  [[ $# = 1 ]] || { echo "$USAGE" 1>&2; return 1; }
  local VERSION=$1

  local RESULT=0
  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local PATCH_DIR="$EPAGES"/Shared/Patches/patch_${VERSION//./_}
  local INSTALL_PATCH=/var/epages/SetupEpages.sh

  [[ -f /etc/debian_version ]] && return 0

  ( set -o posix ; set ) > /tmp/epages-${PATCH_DIR##*/}-prerequisites.log

  echo -e "$INSTALL_PATCH -pre\n" >> /tmp/epages-${PATCH_DIR##*/}-prerequisites.log
  chmod 755 "$INSTALL_PATCH"
  "$INSTALL_PATCH" -pre 2>&1 | tee -a /tmp/epages-${PATCH_DIR##*/}-prerequisites.log
  RESULT=${PIPESTATUS[0]}
  rm -f "$PATCH_DIR"/host-*.mk
  return $RESULT
}

#..............................................................................
# patch_epages
#..............................................................................
patch_epages() {
  local USAGE='
syntax: patch_epages check_prerequistes VERSION
description: patches ePages (if $EPAGES_DO_PATCH != 0)
'
  [[ $# = 1 ]] || { echo "$USAGE" 1>&2; return 1; }

  local RESULT=0
  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local PATCH_DIR="$EPAGES"/Shared/Patches/patch_${VERSION//./_}

  local INSTALL_PATCH=/var/epages/SetupEpages.sh

  echo -e "$INSTALL_PATCH -patch\n" > /tmp/epages-patch_${VERSION//./_}-install.log
  chmod 755 "$INSTALL_PATCH"
  "$INSTALL_PATCH" -patch 2>&1 | tee -a /tmp/epages-patch_${VERSION//./_}-install.log
  RESULT=${PIPESTATUS[0]}
  echo "$INSTALL_PATCH -patch: $RESULT" >> /tmp/epages-patch_${VERSION//./_}-result.log
  echo $RESULT > "$EPAGES"/Shared/Log/epages-patch.result

  return $RESULT
}

#..............................................................................
# patch_repo
#..............................................................................
patch_repo() {
  local USAGE='
syntax: patch_repo VERSION
description: patches the repository (if $EPAGES_DO_PATCH != 0)
'
  [[ $# = 1 ]] || { echo "$USAGE" 1>&2; return 1; }
  local VERSION=$1

  [[ -f /etc/debian_version ]] && return
  [[ $EPAGES_DO_PATCH != 0 ]] || return

  for i in update empty update-archive ; do
    REPO=/etc/yum.repos.d/epages-$i.repo
    MIRROR=/etc/yum.repos.d/mirrors-epages-$i
    j=${i%%-archive}
    [[ -f $REPO ]] || continue
    sed -r "s,/6\.[0-9]+-$j,/${VERSION%.*}-$j," $REPO > /tmp/pid.$$
    if ! cmp $REPO /tmp/pid.$$ >/dev/null 2>&1 ; then
      cat /tmp/pid.$$ > $REPO
      [[ -f $MIRROR ]] && sed -ri "s,/6\.[0-9]+-$j,/${VERSION%.*}-$j," $MIRROR
      rm -rf /var/cache/yum/epages-$i /var/cache/zypp/packages/epages-$i
    fi
    rm -f /tmp/pid.$$
  done
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
