#!/bin/bash

#==============================================================================
# perl
#==============================================================================

fun_perl_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export backup_obsolete_perl set_perl_tree

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# backup_obsolete_perl
#..............................................................................
backup_obsolete_perl() {
  local USAGE='
syntax: backup_obsolete_perl
returns: finds files that existed in the old RPM, but not in the new RPM.
  those files will be packed into a .tgz file
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ -f /etc/debian_version ]] && return

  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local EPAGES_PERL=${EPAGES_PERL:-"$EPAGES"/Perl}

  local PERL_RPM=$(rpm -q epages-perl --qf '[%{NAME}-%{VERSION}\n]' | head -1)

  rpm -q $PERL_RPM --list | sort > /tmp/$PERL_RPM.rpmlist
  find "$EPAGES_PERL"/{*,.??*}/ -type f 2>/dev/null | sort > /tmp/$PERL_RPM.installed

  if [ $(cat /tmp/$PERL_RPM.installed | wc -l) -gt 20 ] ; then
     if [ $(cat /tmp/$PERL_RPM.rpmlist | wc -l) -gt 20 ] ; then
      comm -13 /tmp/$PERL_RPM.rpmlist /tmp/$PERL_RPM.installed > /tmp/$PERL_RPM.norpm 2>/dev/null
      [ -s /tmp/$PERL_RPM.norpm ] && cat /tmp/$PERL_RPM.norpm | tar zcfT - - > "$EPAGES"/$PERL_RPM-norpm.tgz 2>/dev/null
      [ -s /tmp/$PERL_RPM.norpm ] && cat /tmp/$PERL_RPM.norpm | xargs -i{} rm -f "{}" 2>/dev/null
    else
      tar zcf "$EPAGES"/$PERL_RPM.tgz "$EPAGES_PERL"/{*,.??*}/ 2>/dev/null
      rm -rf "$EPAGES_PERL"/{*,.??*}/ 2>/dev/null
    fi
  fi
}

#..............................................................................
# set_perl_tree
#..............................................................................
set_perl_tree () {
  local USAGE='
syntax: set_perl_tree
description: sets path to epages-perl in perl files
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local EPAGES=${EPAGES:-"${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot}
  local EPAGES_PERL=${EPAGES_PERL:-"$EPAGES"/Perl}

  pushd "$EPAGES_PERL"/lib/linux >/dev/null || return 1

  for i in Config.pm Config_heavy.pl ../site_perl/ppm.xml CORE/config.h ; do
    backup_file $i.ept
    sed "s,=EPAGES_PERL=,$EPAGES_PERL,g" $i.ept > $i
  done

  if grep -q ' Perl/' perllocal.pod 2>/dev/null ; then
    cp -p perllocal.pod perllocal.pod.tmp
    sed "s, Perl/, $EPAGES_PERL/,g" perllocal.pod.tmp > perllocal.pod
    rm -f perllocal.pod.tmp
  fi

  popd >/dev/null
  pushd "$EPAGES_PERL"/lib >/dev/null

  for i in $(find . -name .packlist -exec grep \-l '^Perl/' {} \;) ; do
    cp -p $i $i.tmp
    sed "s,^Perl/,$EPAGES_PERL/,g" $i.tmp > $i
    rm -f $i.tmp
  done
  popd >/dev/null
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
