#!/bin/bash

#==============================================================================
# package manager
#==============================================================================

fun_pkg_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL EPAGES_ARCH

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export pkg_instance pkg_installed

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# pkg_instance
#..............................................................................
pkg_instance() {
  local USAGE='
syntax: pkg_instance NUMBER
NUMBER: 0 - remove last version of package
        1 - install the first time
        else - update (depending on the number of versions installed)
returns: first  - if NUMBER==1
         last   - if NUMBER==0
         update - else
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  [[ $1 = 1 ]] && { echo first; return; }
  [[ $1 = 0 ]] && { echo last; return; }
  echo update
}

#..............................................................................
# pkg_installed
#..............................................................................
pkg_installed() {
  local USAGE='
syntax: pkg_installed [-v|-V] [-and|-or] PKG1 PKG2...
arguments:
         -v: verbose output
         -V: outputs version number (works only with 1 PKG)
description: checks if epages-PKG is installed
returns: success (0) - if all PKG1 PKG2... are installed
         success (0) - if one of PKG1 PKG2... is installed and -or is set
         failure (1) - else
'
  local pkg version SHOW_VERSION
  local OR=0
  local QUIET=--quiet
  while [[ $1 =~ ^- ]] ; do
    case $1 in
     -v) QUIET= ; shift ;;
     -V) SHOW_VERSION=1 ; shift ;;
     -or) OR=1 ; shift ;;
     -and) shift ;;
     -*)  echo "$USAGE" 1>&2; return 1 ;;
     *) break ;;
    esac
  done

  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }
  local all=$#
  local found=0

  for pkg in $@ ; do
    if [[ $pkg = pubkey && $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
      rpm -qa | grep -qi gpg-pubkey-b1d28320- || continue
    elif [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
      rpm $QUIET -q epages-$pkg || continue
      [[ -n $SHOW_VERSION ]] && version=$(rpm -q epages-$pkg --qf '[%{VERSION}-%{RELEASE}\n]' | sort -V | tail -1)
    elif [[ $pkg = pubkey ]] ; then
      apt-key list | grep -qi '/B1D28320 ' || continue
    elif [[ -n $QUIET ]] ; then
      dpkg --status epages-$pkg >/dev/null 2>&1 || continue
      [[ -n $SHOW_VERSION ]] && version=$(dpkg-query --show -f '${Version}\n' epages-$pkg)
    else
      dpkg --status epages-$pkg || continue
      [[ -n $SHOW_VERSION ]] && version=$(dpkg-query --show -f '${Version}\n' epages-$pkg)
    fi
    found=$(($found + 1))
  done
  [[ -n $SHOW_VERSION ]] && echo -n $version
  [[ $found = $all || ( $found -gt 0 && $OR = 1 ) ]] && return 0
  return 1
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
