#!/bin/bash

#==============================================================================
# WebInterface.conf functions
#==============================================================================

fun_wi_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file repair_number

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export get_ip_info get_host_ip get_host_name guess_addr_type my_ips
export normalize_IPv6_address resolve_host_info

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export set_wi set_wi_sections set_wi_exception_host 
export set_wi_as set_wi_rr set_wi_as2rr set_wi_rr2as
export set_wi_xml set_wi_xml_as set_wi_xml_asc set_wi_xml_rr

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# set_wi_as
#..............................................................................
set_wi_as () {
  local USAGE='
syntax: set_wi_as WI_CONF
description: adds AS ports to [1.2.3.4]APPLICATIONSERVER in WebInterface.conf
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  . /etc/sysconfig/epages6
  EPAGES_INSTALL_AS_FIRST_PORT=$(repair_number "$EPAGES_INSTALL_AS_FIRST_PORT" 1024.. 10045)

  local AS_IP=$(get_host_ip AS)

  local AS_PORTS
  AS_PORTS=$($INI_CONFIG -f "$WI_CONF" -s "$AS_IP" -k APPLICATIONSERVER -g 2>/dev/null)
  AS_PORTS=(${AS_PORTS//,/ })

  # add $AS_PORTS to APPLICATIONSERVER in section $AS_IP up to $AS_NUMBER

  local AS_NUMBER=2
  while [[ ${#AS_PORTS[@]} -lt $AS_NUMBER ]] ; do
    while [[ " "${AS_PORTS[@]}" " == *" "$EPAGES_INSTALL_AS_FIRST_PORT" "* ]] ; do
      let EPAGES_INSTALL_AS_FIRST_PORT+=1
    done
    AS_PORTS=(${AS_PORTS[@]} $EPAGES_INSTALL_AS_FIRST_PORT)
  done

  local ALL_PORTS=${AS_PORTS[@]}
  $INI_CONFIG -f "$WI_CONF" -s "$AS_IP" -k APPLICATIONSERVER -v "${ALL_PORTS// /,}" -i 2>/dev/null
}

#..............................................................................
# set_wi_rr
#..............................................................................
set_wi_rr () {
  local USAGE='
syntax: set_wi_rr WI_CONF
description: adds RR ports to [1.2.3.4]MONITOR/MESSAGECENTER/REQUESTROUTER in WebInterface.conf
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  . /etc/sysconfig/epages6

  local RR_IP=$(get_ip_info RR)
  RR_IP=${RR_IP:-$EPAGES_INSTALL_RR_DEFAULT}
  RR_IP=${RR_IP:-$(get_host_ip RR)}

  EPAGES_INSTALL_RR_PORT=$(repair_number "$EPAGES_INSTALL_RR_PORT" 1024.. 10043)
  EPAGES_INSTALL_RR_MON_PORT=$(repair_number "$EPAGES_INSTALL_RR_MON_PORT" 1024.. 10041)
  EPAGES_INSTALL_RR_MC_PORT=$(repair_number "$EPAGES_INSTALL_RR_MC_PORT" 1024.. 10042)

  local WI_KEY WI_VALUE
  WI_KEY=(MONITOR MESSAGECENTER REQUESTROUTER)
  WI_VALUE=($EPAGES_INSTALL_RR_MON_PORT $EPAGES_INSTALL_RR_MC_PORT $EPAGES_INSTALL_RR_PORT)

  for i in 0 1 2 ; do
    $INI_CONFIG -f "$WI_CONF" -s "$RR_IP" -k "${WI_KEY[$i]}" -v "${WI_VALUE[$i]}" -i 2>/dev/null
  done
}

#..............................................................................
# set_wi_as2rr
#..............................................................................
set_wi_as2rr () {
  local USAGE='
syntax: set_wi_as2rr WI_CONF
description: updates DEDICATED list with AS
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  . /etc/sysconfig/epages6

  local AS_IP=$(get_host_ip AS)

  if [[ -z $EPAGES_INSTALL_RR_DEDICATED ]] ; then
    EPAGES_INSTALL_RR_DEDICATED=$AS_IP
    $INI_CONFIG -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_RR_DEDICATED -v "$AS_IP" -i 2>/dev/null
  elif [[ ",$EPAGES_INSTALL_RR_DEDICATED," != *,$AS_IP,* ]] ; then
    EPAGES_INSTALL_RR_DEDICATED="$EPAGES_INSTALL_RR_DEDICATED,$AS_IP"
    $INI_CONFIG -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_RR_DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
  fi

  if [[ -n $EPAGES_INSTALL_RR_DEDICATED && -n $EPAGES_INSTALL_RR_DEFAULT ]] ; then
    $INI_CONFIG -f "$WI_CONF" -s "$EPAGES_INSTALL_RR_DEFAULT" -k DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
  fi
}

#..............................................................................
# set_wi_rr2as
#..............................................................................
set_wi_rr2as () {
  local USAGE='
syntax: set_wi_rr2as WI_CONF
description: updates DEDICATED list for RR
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  . /etc/sysconfig/epages6

  local RR_IP=${EPAGES_INSTALL_RR_DEFAULT:-$(get_host_ip RR)}

  if [[ -z $EPAGES_INSTALL_RR_DEFAULT ]] ; then
    EPAGES_INSTALL_RR_DEFAULT=$RR_IP
    $INI_CONFIG -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_RR_DEFAULT -v "$RR_IP" -i 2>/dev/null
  fi

  if [[ -n $EPAGES_INSTALL_RR_DEDICATED ]] ; then
    $INI_CONFIG -f "$WI_CONF" -s "$RR_IP" -k DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
  fi
}

#..............................................................................
# set_wi_sections
#..............................................................................
set_wi_sections () {
  local USAGE='
syntax: set_wi_sections WI_CONF
description: adds (empty) sections with local IPs to WebInterface.conf
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ADDR NET
  for ADDR in $(my_ips) ; do
    if [[ $(guess_addr_type "$ADDR") = IPv6 ]] ; then
      ADDR=$(normalize_IPv6_address $ADDR)
      NET=
    else
      $INI_CONFIG -f "$WI_CONF" -s "::ffff:$ADDR" -i 2>/dev/null
      NET=${ADDR%.*}.'*'
    fi
    $INI_CONFIG -f "$WI_CONF" -s "$ADDR" -i 2>/dev/null
    $INI_CONFIG -f "$WI_CONF" -s "$NET" -i 2>/dev/null
  done
}

#..............................................................................
# set_wi_exception_host
#..............................................................................
set_wi_exception_host () {
  local USAGE='
syntax: set_wi_exception_host WI_CONF
description: adds WS IP/host list to [URLRewrite]ExceptionHost in WebInterface.conf
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF=$1
  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local EXCEPTION_HOST=" "
  for i in $($INI_CONFIG -f "$WI_CONF" -s URLRewrite -k ExceptionHost -g 2>/dev/null) \
  $(resolve_host_info) ; do
    [[ ${i%:*} != $i && ${i%[*} = $i ]] && i="[$i]"
    q=$(echo "$i" | sed 's/\([].[]\)/\\\1/'g)
    [[ ${EXCEPTION_HOST// $q /} = $EXCEPTION_HOST ]] || continue
    EXCEPTION_HOST="$EXCEPTION_HOST$i "
  done
  EXCEPTION_HOST="${EXCEPTION_HOST% }"
  $INI_CONFIG -f "$WI_CONF" -s URLRewrite -k ExceptionHost -v "${EXCEPTION_HOST# }" -i 2>/dev/null
}

#..............................................................................
# set_wi
#..............................................................................
set_wi () {
  local USAGE='
syntax: set_wi {WHAT...}
description: edits WHAT in WebInterface.conf
WHAT is one/more of:
  - AS
  - AS2RR
  - RR
  - RR2AS
  - SECTIONS
  - EXCEPTION_HOST
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_CONF="$EPAGES_CONFIG"/WebInterface.conf
  backup_file "$WI_CONF".ept
  [[ -f "$WI_CONF" ]] || {
    echo "ERROR: $BASH_SOURCE: set_wi:" 1>&2; ls "$WI_CONF"; return 1
  }

  local WHAT
  for WHAT in $@ ; do
    eval set_wi_$(tr "[A-Z]" "[a-z]" <<<$WHAT) "$WI_CONF"
  done
}

#..............................................................................
# set_wi_xml_as
#..............................................................................
set_wi_xml_as () {
  local USAGE='
syntax: set_wi_xml_as WI_XML
description: sets host/ports for <appserver> in ServerConfig.xml
'

  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_XML=$1

  . /etc/sysconfig/epages6
  local AS_PORTS=$(repair_number "$EPAGES_INSTALL_AS_FIRST_PORT" 1024.. 10045)
  AS_PORTS="$AS_PORTS-$(($AS_PORTS + 1))"

  local AS_IP=$(get_host_ip AS)
  export AS_IP AS_PORTS

  egrep -v "<appserver\b.*\b(localhost|$AS_IP)\b" $WI_XML \
  | PERL5LIB= /usr/bin/perl -ne '
print $_;
if (/<appservers>/ and not $Again) {
  print "        <appserver host=\"$ENV{AS_IP}\" ports=\"$ENV{AS_PORTS}\" maxmemory=\"200\" />\n";
  $Again = 1;
}
' > /tmp/ServerConfig.xml.~$$~ && cat /tmp/ServerConfig.xml.~$$~ > $WI_XML
  rm -f /tmp/ServerConfig.xml.~$$~
}

#..............................................................................
# set_wi_xml_rr
#..............................................................................
set_wi_xml_rr () {
  local USAGE='
syntax: set_wi_xml_rr WI_XML
description: sets host/ports for <requestrouter> in ServerConfig.xml
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_XML=$1

  . /etc/sysconfig/epages6

  local RR_IP=$(get_ip_info RR)
  RR_IP=${RR_IP:-$EPAGES_INSTALL_RR_DEFAULT}
  RR_IP=${RR_IP:-$(get_host_ip RR)}

  RR_PORT=$(repair_number "$EPAGES_INSTALL_RR_PORT" 1024.. 10043)
  RR_MON_PORT=$(repair_number "$EPAGES_INSTALL_RR_MON_PORT" 1024.. 10041)
  RR_MC_PORT=$(repair_number "$EPAGES_INSTALL_RR_MC_PORT" 1024.. 10042)

  sed -ri \
  "s,(<requestrouter name=.DefaultRequestRouter. host)=.localhost. .*,\1='$RR_IP' port='$RR_PORT' monitor_port='$RR_MON_PORT' mc_port='$RR_MC_PORT' />," \
  $WI_XML
}

#..............................................................................
# set_wi_xml_asc
#..............................................................................
set_wi_xml_asc () {
  local USAGE='
syntax: set_wi_xml_asc WI_XML
description: sets host/ports for <server> in ServerConfig.xml
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_XML=$1

  . /etc/sysconfig/epages6

  local RR_IP=$(get_ip_info RR)
  RR_IP=${RR_IP:-$EPAGES_INSTALL_RR_DEFAULT}
  RR_IP=${RR_IP:-$(get_host_ip RR)}

  sed -ri \
  "s,(<server host)=.localhost. .*,\1='$RR_IP' port='17277'/>," \
  $WI_XML
}

#..............................................................................
# set_wi_xml
#..............................................................................
set_wi_xml () {
  local USAGE='
syntax: set_wi_xml {WHAT...}
description: edits WHAT in ServerConfig.xml
WHAT is one/more of:
  - AS
  - ASC
  - RR
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local WI_XML="$EPAGES_CONFIG"/ServerConfig.xml
  backup_file "$WI_XML".ept
  [[ -f "$WI_XML" ]] || {
    echo "ERROR: $BASH_SOURCE: set_wi_xml:" 1>&2; ls "$WI_XML"; return 1
  }

  local WHAT
  for WHAT in $@ ; do
    eval set_wi_xml_$(tr "[A-Z]" "[a-z]" <<<$WHAT) "$WI_XML"
  done
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
