#!/bin/bash

#==============================================================================
# epages-appsrv.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service
export etc_default enable_ipv6 set_local_ips

. $(dirname $0)/fun-wi.sh
export set_wi set_wi_xml

. $(dirname $0)/fun-account.sh
export chown_epages chgrp_epages

. $(dirname $0)/fun-pkg.sh
export pkg_instance

. $(dirname $0)/fun-host.sh
export get_host_name

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-appsrv-post
#..............................................................................
epages-appsrv-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-appsrv-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-appsrv package'
    echo 'example: epages-appsrv-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  update_epages_environment -i "$1"

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  [[ -f $ETC_DEFAULT ]] || { ls -l $ETC_DEFAULT; exit 1; } 
  . $ETC_DEFAULT

  local HAS_ASC=$(strings $EPSERVICE | grep -i cacheserver)
  if [[ -n $HAS_ASC ]] ; then
    set_wi EXCEPTION_HOST
    [[ $INSTANCE = first ]] && set_wi_xml AS
  else
    set_wi AS AS2RR EXCEPTION_HOST SECTIONS
    set_local_ips AS
  fi

  enable_ipv6

  for i in $(grep -l BackupHost=localhost "$EPAGES_CONFIG"/Database.d/*.{conf,initial}) ; do
    sed -i "s/\(BackupHost\)=localhost/\1=$(get_host_name AS)/" $i
  done

  chown_epages AS "$EPAGES_CONFIG/"

  if [[ $INSTANCE = first ]] ; then
    chgrp_epages -R WS "$EPAGES_LOG/" "$EPAGES_WEBROOT/"
    chown_epages -R AS "$EPAGES_LOG/"
    chown_epages -R AS_RUN "$EPAGES_WEBROOT/"
  else
    chgrp_epages WS "$EPAGES_WEBROOT/"
    chown_epages AS_RUN "$EPAGES_WEBROOT/"
    chgrp_epages -R WS "$EPAGES_WEBROOT"/ImageDB/
    chown_epages -R AS_RUN "$EPAGES_WEBROOT"/ImageDB/
  fi

  epages_service start AS

  return 0
}

#..............................................................................
# epages-appsrv-preun
#..............................................................................
epages-appsrv-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-appsrv-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-appsrv package'
    echo 'example: epages-appsrv-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  epages_service stop AS

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  post)  epages-appsrv-post  $2 $3 ; exit $? ;;
  preun) epages-appsrv-preun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
