#!/bin/bash

#==============================================================================
# epages-ascsrv.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service
export etc_default enable_ipv6

. $(dirname $0)/fun-wi.sh
export set_wi set_wi_xml

. $(dirname $0)/fun-pkg.sh
export pkg_instance

. $(dirname $0)/fun-mysql.sh
export set_mysql_connection get_mysql_connection

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-ascsrv-post
#..............................................................................
epages-ascsrv-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-ascsrv-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-ascsrv package'
    echo 'example: epages-ascsrv-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  update_epages_environment -i "$1"

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  [[ -f $ETC_DEFAULT ]] || { ls -l $ETC_DEFAULT; exit 1; } 
  . $ETC_DEFAULT

  local DB_CONF="$EPAGES_CONFIG/Database.d/ASPoolDB.conf"
  if [[ -f  $DB_CONF.sample.initial ]] ; then
    cp -p $DB_CONF.sample.initial $DB_CONF.initial
  fi
  [[ ! -f $DB_CONF ]] && cp -p $DB_CONF.initial $DB_CONF

  set_wi EXCEPTION_HOST
  [[ $INSTANCE = first ]] && set_wi_xml ASC

  enable_ipv6
  epages_service start ASC

  return 0
}

#..............................................................................
# epages-ascsrv-preun
#..............................................................................
epages-ascsrv-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-ascsrv-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-ascsrv package'
    echo 'example: epages-ascsrv-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  epages_service stop ASC

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  post)  epages-ascsrv-post  $2 $3 ; exit $? ;;
  preun) epages-ascsrv-preun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
