#!/bin/bash

#==============================================================================
# epages-j.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-account.sh
export add_user chown_epages

. $(dirname $0)/fun-epages.sh
export create_etc_default update_epages_environment link_epages link_etc

. $(dirname $0)/fun-j.sh
export install_epagesj_conf start_epagesj epagesj_drop_cores
export update_epagesj_conf update_epagesj_conf_xml

. $(dirname $0)/fun-java.sh
export start_mongodb test_version

. $(dirname $0)/fun-pkg.sh
export pkg_instance pkg_installed

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-j-pre
#..............................................................................
epages-j-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-j-pre INSTANCE RPM_VERSION'
    echo 'description: pre-uninstallation tasks of epages-j package'
    echo 'example: epages-j-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  if [[ $INSTANCE = first ]] ; then
    add_user -u AS -g AS -G WS
    add_user -u J -g AS -G WS
  fi

  [[ -n $EPAGES_SKIP_SPEC ]]  && return 0

  for i in epages-solr epages-solr-slave epagesj ; do
    /etc/init.d/$i status >/dev/null 2>&1 && /etc/init.d/$i stop
  done

  local EPAGESJ="${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot/j
  rm -rf "$EPAGESJ"/work/* /tmp/epagesj_prev_version

  if cd "$EPAGESJ" 2>/dev/null ; then
    rm -rf rpmsave ; mkdir -m 755 -p rpmsave
    tar cf - */conf 2>/dev/null | ( cd rpmsave ; tar xf - )
    cp -rp "$EPAGESJ"/conf rpmsave/conf 2>/dev/null
  fi

  # remember previous epages-j version
  pkg_installed -V j | tail -1 | sed 's,-.*,,' > /tmp/epagesj_prev_version

  epagesj_drop_cores -b $VERSION || return 0

  [[ $INSTANCE = first ]] || echo "rm -rf $EPAGESJ/cores/*"
  rm -rf "$EPAGESJ"/cores/*

  return 0
}

#..............................................................................
# epages-j-post
#..............................................................................
epages-j-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-j-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-j package'
    echo 'example: epages-j-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  create_etc_default epagesj.ept

  [[ -z $EPAGES_SKIP_SPEC ]] && update_epages_environment -i "$1"

  link_epages $RPM_INSTALL_PREFIX

  link_etc default

  [[ $INSTANCE = first ]] && enable_ipv6

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  [[ -f $ETC_DEFAULT ]] || { ls -l $ETC_DEFAULT; exit 1; } 
  . $ETC_DEFAULT

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  local EPAGESJ="${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot/j
  local CONF="$EPAGESJ"/conf/epagesj.conf
  local SAMPLE="$EPAGESJ"/conf/epagesj.sample.conf

  [[ -f "$SAMPLE" ]] && /bin/cp -p "$SAMPLE" "$CONF"

  [[ $INSTANCE = first ]] && install_epagesj_conf || update_epagesj_conf
  [[ $INSTANCE = first ]] || update_epagesj_conf_xml

  chown_epages -R J "$EPAGESJ"

  local PREV_VERSION=0
  [[ -f /tmp/epagesj_prev_version ]] && PREV_VERSION=$(cat /tmp/epagesj_prev_version)
  PREV_VERSION=${PREV_VERSION//[^0-9.]/}
  PREV_VERSION=${PREV_VERSION:-0}

  if [[ $INSTANCE = first || -z $SKIP_START_JAVA ]] ; then
    start_mongodb
    local START_SERVICE=
    # if version to install >= 6.17.14 and initial installation or previous version < 6.17.14
    if test_version $VERSION -ge 6.17.14 && test_version $PREV_VERSION -lt 6.17.14 ; then
      # stop service epagesj if it can't connect to epagesj mysqldb
      if ! /var/epages/CheckMySQLConnection.sh epagesj >/dev/null 2>&1 ; then
        START_SERVICE="epages-solr epages-solr-slave"
        echo "-> service epagesj was stopped: no connection to epagesj MySQL database"
        echo "-> epagesj will be started after epages $VERSION is installed"
      fi
    fi
    start_epagesj $START_SERVICE
  fi

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)   epages-j-pre $2 $3 ; exit $? ;;
  post)  epages-j-post $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
