#!/bin/bash

#==============================================================================
# epages-mysqld.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-pkg.sh
export pkg_instance

. $(dirname $0)/fun-linux.sh
export chkconfig_do

. $(dirname $0)/fun-epages.sh
export etc_default

. $(dirname $0)/fun-mysql.sh
export my_cnf

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-mysqld-post
#..............................................................................
epages-mysqld-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-mysqld-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-mysqld package'
    echo 'example: epages-mysqld-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ $INSTANCE = first ]] || return 0

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  [[ -f $(readlink -f "$ETC_DEFAULT") ]] && . "$ETC_DEFAULT"

  local EPAGES_MYSQLD=$(mysqld_name)
  [[ -z $EPAGES_MYSQLD ]] && return 0

  if [[ -z $EPAGES_SKIP_SPEC ]] ; then
    enable_ipv6

    local INI_CONFIG=/var/epages/ini-config.sh
    local DATABASE_CONF=${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}/Database.d

    # if EPAGES_ENABLE_IPv6 is set then define key bind-address=::
    # in section [mysqld] (if exist) for any /etc/my.cnf
    # if nameserver is not IPv6, mysqld will fail starting IPv6
    if [[ $INSTANCE = first && $EPAGES_ENABLE_IPv6 = 1 ]] \
       && grep -q '^nameserver.*:' /etc/resolv.conf ; then
      for CONF in $(my_cnf) $(grep '^\s*MySQLConfigFile\s*=' $DATABASE_CONF/*.conf \
         2>/dev/null | sed 's/.*MySQLConfigFile[[:space:]]*=[[:space:]]*//;s/["''"]//g;')
      do
        $INI_CONFIG -f "$CONF" -s mysqld -g | grep -q . || continue
        $INI_CONFIG -f "$CONF" -s mysqld -k bind-address -v '::' -i
      done
    fi
  fi  

  chkconfig_do --del $EPAGES_MYSQLD
  local RUN_SCRIPT=/etc/init.d/$EPAGES_MYSQLD
  if [[ -e $RUN_SCRIPT ]] ; then
    if ! grep -q 'chkconfig: 235' $RUN_SCRIPT ; then
      sed -i 's/\(chkconfig:\) -/\1 235/' $(readlink -qf $RUN_SCRIPT)
    fi
    chmod 755 $RUN_SCRIPT
  fi
  chkconfig_do --add $EPAGES_MYSQLD

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  service $EPAGES_MYSQLD start
  "${RPM_INSTALL_PREFIX:-/srv/epages}"/eproot/Setup/SetupMySQL.sh

  return $?
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  post) epages-mysqld-post  $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
