#!/bin/bash

#==============================================================================
# epages-patch.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_DO_PATCH

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-pkg.sh
export pkg_instance

. $(dirname $0)/fun-patch.sh
export check_prerequistes patch_epages patch_repo

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-patch-pre
#..............................................................................
epages-patch-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-patch-pre INSTANCE RPM_VERSION'
    echo 'description: pre-uninstallation tasks of epages-patch package'
    echo 'example: epages-patch-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ $INSTANCE = update ]] || return 0

  check_prerequistes $VERSION

  return $?
}

#..............................................................................
# epages-patch-post
#..............................................................................
epages-patch-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-patch-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-patch package'
    echo 'example: epages-patch-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ $INSTANCE = update ]] || return 0
  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  patch_epages $VERSION

  return $?
}

#..............................................................................
# epages-patch-patched
#..............................................................................
epages-patch-patched() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-patch-postun INSTANCE RPM_VERSION'
    echo 'description: postun-uninstallation tasks of epages-patch package'
    echo 'example: epages-patch-postun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  patch_repo $VERSION

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)     epages-patch-pre     $2 $3 ; exit $? ;;
  post)    epages-patch-post    $2 $3 ; exit $? ;;
  patched) epages-patch-patched $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
