#!/bin/bash

#==============================================================================
# epages-perl.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service

. $(dirname $0)/fun-pkg.sh
export pkg_instance

. $(dirname $0)/fun-perl.sh
export backup_obsolete_perl set_perl_tree

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-perl-pre
#..............................................................................
epages-perl-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-pre INSTANCE RPM_VERSION'
    echo 'description: pre-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ $INSTANCE = first ]] || return 0
  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  epages_service stop AS
  [[ $INSTANCE = first ]] || backup_obsolete_perl

  return 0
}

#..............................................................................
# epages-perl-post
#..............................................................................
epages-perl-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-perl package'
    echo 'example: epages-perl-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ $INSTANCE = first ]] || return 0

  [[ -z $EPAGES_SKIP_SPEC ]] && update_epages_environment -i "$1"

  set_perl_tree

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  epages_service start AS

  if [ -f /etc/SuSE-release ] ; then
    cd /usr/lib
    [ -e libssl.so.6 ] || ln -s libssl.so.0.9.8 libssl.so.6
    [ -e libcrypto.so.6 ] || ln -s libcrypto.so.0.9.8 libcrypto.so.6
  fi

  return 0
}

#..............................................................................
# epages-perl-preun
#..............................................................................
epages-perl-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  epages_service stop AS

  return 0
}

#..............................................................................
# epages-perl-postun
#..............................................................................
epages-perl-postun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-perl-postun INSTANCE RPM_VERSION'
    echo 'description: postun-uninstallation tasks of epages-perl package'
    echo 'example: epages-perl-postun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  [[ $INSTANCE = last ]] || epages_service start AS

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)     epages-perl-pre    $2 $3 ; exit $? ;;
  post)    epages-perl-post   $2 $3 ; exit $? ;;
  preun)   epages-perl-preun  $2 $3 ; exit $? ;;
  postun)  epages-perl-postun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
