#!/bin/bash

#==============================================================================
# epages-reqsrv.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service
export etc_default enable_ipv6 set_local_ips

. $(dirname $0)/fun-wi.sh
export set_wi set_wi_xml

. $(dirname $0)/fun-pkg.sh
export pkg_instance

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-reqsrv-post
#..............................................................................
epages-reqsrv-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-reqsrv-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-reqsrv package'
    echo 'example: epages-reqsrv-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  update_epages_environment -i "$1"

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  [[ -f $ETC_DEFAULT ]] || { ls -l $ETC_DEFAULT; exit 1; } 
  . $ETC_DEFAULT

  local HAS_ASC=$(strings $EP_RR | grep -i cacheserver)

  if [[ -n $HAS_ASC ]] ; then
    set_wi EXCEPTION_HOST
    [[ $INSTANCE = first ]] && set_wi_xml RR
  else
    set_wi RR RR2AS EXCEPTION_HOST SECTIONS
    set_local_ips RR
  fi

  enable_ipv6
  epages_service start RR

  return 0
}

#..............................................................................
# epages-reqsrv-preun
#..............................................................................
epages-reqsrv-preun() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-reqsrv-preun INSTANCE RPM_VERSION'
    echo 'description: preun-uninstallation tasks of epages-reqsrv package'
    echo 'example: epages-reqsrv-preun 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  epages_service stop RR

  return 0
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  post)  epages-reqsrv-post  $2 $3 ; exit $? ;;
  preun) epages-reqsrv-preun $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
