#!/bin/bash

#==============================================================================
# epages-setup.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# recognized variables
#------------------------------------------------------------------------------

export EPAGES_INSTALL_VERSION

[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export etc_default

. $(dirname $0)/fun-pkg.sh
export pkg_instance

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-setup-post
#..............................................................................
epages-setup-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-setup-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-setup package'
    echo 'example: epages-setup-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=${EPAGES_INSTALL_VERSION:-$2}

  [[ $INSTANCE = first ]] || return 0
  [[ -n $EPAGES_SKIP_SPEC ]] && return 0

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  [[ -f $ETC_DEFAULT ]] || { ls -l $ETC_DEFAULT; exit 1; } 

  /var/epages/SetupEpages.sh
  local RESULT=$?
  echo $RESULT > "${EPAGES:-/srv/epages/eproot}"/Shared/Log/epages-setup.result
  return $RESULT
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  post) epages-setup-post $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
