#!/bin/bash

BN=${0##*/}

usage() {
  ( echo "Usage:"
    echo "  $BN -f file { [-s section] [-k key] } [-v value [-r subst] | -g|-G] [-i] [-p {k|s|ks}]"
    echo "Options:"
    echo "  -f: INI file name"
    echo "  -s: section (REGEX) as in [section]"
    echo "  -k: key (REGEX) as in key="
    echo "  -v: set key to value"
    echo "  -r: replace subst (REGEX) with value in key"
    echo "  -g: get key value (first occurrence)"
    echo "  -G: get key value (first occurrence, unquoted)"
    echo "  -i: edit file in place"
    echo "  -p: plain key and/or section (do not quote)" ) 1>&2
  exit 1
}

[ $# -eq 0 ] && usage

unset FILE SECTION KEY VALUE SUBST GET GET_UNQUOTED INPLACE PLAIN
while getopts f:s:k:v:r:gGip: c ; do
  case $c in
    f) FILE=$OPTARG ;;
    s) SECTION=$OPTARG ;;
    k) KEY=$OPTARG ;;
    v) VALUE=$OPTARG ;;
    r) SUBST=$OPTARG ;;
    g) GET=1 ;;
    G) GET=1 ; GET_UNQUOTED=1 ;;
    i) INPLACE=1 ;;
    p) PLAIN=$OPTARG ;;
    *) usage ;;
  esac
done
export GET SUBST SECTION KEY

[ -z "$FILE" ] && usage
[ -z "$GET" -a -z "$KEY" -a -z "$SECTION" ] && usage
[ -n "$VALUE" -a -n "$GET" ] && usage
[ -z "$VALUE" -a -n "$SUBST" ] && usage
ls "$FILE" >/dev/null || usage
[ -n "$GET" ] && unset INPLACE

EPOCH=`date "+%s"`
[ -z "$INPLACE" ] && FROM="$FILE" || { FROM="$FILE".~$EPOCH~; cp -p "$FILE" "$FROM"; }

if [ -n "$GET" -a -z "$KEY" -a -z "$SECTION" ] ; then
  awk -F\[ '/^[\t ]*\[[^]]*\][\t ]*$/{sub(/\].*/,"",$2);print $2}' "$FROM"
  exit 0
fi

PATH=/usr/bin:/bin:$PATH
PERL5LIB=
type perl >/dev/null || exit 1
QKEY="$(echo "$KEY" | sed 's,\\,\\\\,g;s,/,\\/,g')"
[[ $PLAIN != ${PLAIN/k/} ]] && QKEY=$KEY
QSECTION="$(echo "$SECTION" | sed 's,\\,\\\\,g;s,/,\\/,g;s,\*,\\*,')"
[[ $PLAIN != ${PLAIN/s/} ]] && QSECTION=$SECTION

perl -e '$/ = undef; sub nl { $x = $_[0]; $x =~ s/\n/.Nin1L/g; return "=$x\n";};
while(<>) { if($ENV{GET} && !$ENV{KEY}) {s/((^|\n)\s*[#;].*|[#;][^\042\047]*$)//g;};
            s/\r//g; s/=<<HERE\n(.+?)\nHERE\n/nl($1)/ges; print;}' \
< "$FROM" | awk -F= -v section="$SECTION" -v key="$KEY" -v set="$VALUE" -v get="$GET" -v gu="$GET_UNQUOTED" -v subst="$SUBST" '
BEGIN{
  i = 0; spc = ""; spc_defd = 0;
  if (section ~ /^$/) i = 1;
} {
  # get leading space for keys if not yet defined
  if (!spc_defd && $1 !~ /^[\t ]*[;#]/ && $2 ~ /./ && $2 ~ /[^\t ]/) { spc = $2; spc_defd = 1; sub(/[^\t ].*/, "", spc); }
  # if not section line and section found and ( key is undef or key matches )
  if ($0 !~ /^[\t ]*(\[|$)/ && i > 0 && (key ~ /^$/ || ( key ~ /./ && /^[\t ]*'"$QKEY"'[\t ]*=/))) {
    # print line if get is defined but key not
    if (get ~ /./ && key ~ /^$/) { print $0; }
    # if get is defined (and key matches) print value
    else if (get ~ /./) { sub(/^[^=]*=[\t ]*/,"");
      if (gu ~ /./) { sub(/^[\042\047]/,""); sub(/[\042\047]$/,""); }
      print $0; exit 0;
    }
    # substitute SUBST in set
    else if (subst ~ /./) { gsub(/'"$SUBST"'/, set); print $0; }
    # print everything (used to add a new section)
    else if (key ~ /^$/) { print $0; }
    # set key to new value
    else { printf "%s=%s%s\n", $1, spc, set; }
    i = 2;
  } else {
    if (section ~ /./) {
      # set section found if section matches
      if (/^[\t ]*\['"$QSECTION"'\]/) { i = 1; }
      # non-matching section
      else if (/^[\t ]*\[/) {
        # add new key to prior section
        if (get ~ /^$/ && key ~ /./ && i == 1 && subst ~ /^$/) printf "%s=%s%s\n", key, spc, set;
        # unset section found
        if (i > 0) i = -i;
      }
    }
    # print section line (if not yet done)
    if (get ~ /^$/) print $0;
  }
} END {
  # if section is empty and key not found
  if (section ~ /^$/ && i != 2) i = 0;
  if (get ~ /^$/ && (i == 0 || i == 1) && subst ~ /^$/) {
    # add new section if defined
    if (section ~ /./ && i == 0) printf "[%s]\n", section;   
    # add new key if defined
    if (key ~ /./) printf "%s=%s%s\n", key, spc, set;
  }
}' | sed 's,\.Nin1L,\n,g' | { [ -z "$INPLACE" ] && cat - || cat - > "$FILE"; }

[ -n "$INPLACE" ] && rm -f "$FROM"
exit 0
