#!/bin/env bash

BN=${0##*/}
DN=$(cd $(dirname $0) && pwd)

usage() {
  ( echo "Usage:"
    echo "  $BN -s epages6 -t epages6.ept [-i]"
    echo "Options:"
    echo "  -s: file to change (e.g.: /etc/default/epages6)"
    echo "  -t: new template file (e.g.: /srv/epages/etc/default/epages6.ept)"
    echo "  -i: edit file to change in place" ) 1>&2
  exit 1
}

[[ $# -eq 0 ]] && usage

unset ETC_DEFAULT DEFAULT_EPT INPLACE
while getopts s:t:i c ; do
  case $c in
    s) ETC_DEFAULT=$OPTARG ;;
    t) DEFAULT_EPT=$OPTARG ;;
    i) INPLACE=1 ;;
    *) usage ;;
  esac
done

[[ -z $ETC_DEFAULT || -z $DEFAULT_EPT ]] && usage
ls "$DEFAULT_EPT" >/dev/null || usage

PATH=$DN:/var/epages:$PATH
for i in functions.sh ini-config.sh ; do
  type $i >/dev/null || exit $?
done

FROM=/tmp/default.~$$~

# set existing environment

. functions.sh
ETC_DEFAULT=$(readlink -f "$ETC_DEFAULT")
[[ -f "$ETC_DEFAULT" ]] && . "$ETC_DEFAULT"
cp -p "$DEFAULT_EPT" "$FROM" || exit 1
touch "$FROM".value || exit 1
set | awk -F= '{print $1}' > "$FROM".set

# foreach ENVVAR in $DEFAULT_EPT set value in $FROM
awk -F= '/^\s*\w.*=/{print $1}' "$DEFAULT_EPT" | while read ENVVAR ; do
  ENVVAR_KEY=$ENVVAR
  if [[ $ENVVAR = EPAGES_HTTPD_ROOT && -f "$HTTPD_ROOT"/bin/httpd.sh ]] ; then
    grep -q "^$ENVVAR$" "$FROM".set || {
      grep -q "^HTTPD_ROOT$" "$FROM".set && ENVVAR=HTTPD_ROOT
    }
  fi
  grep -q "^$ENVVAR$" "$FROM".set || continue # new variable
  ENV=$(eval echo '$'$ENVVAR)
  unset SHORTER key value
  if [[ ${ENV#*/} != $ENV ]] ; then # ENV has slash
    while read key value ; do
      [[ ${ENV#$value} != $ENV ]] && SHORTER="$key ${ENV#$value}"
    done < "$FROM".value
    echo "$ENVVAR $ENV" >> "$FROM".value
  fi
  [[ -n $SHORTER ]] && ENV="\$${SHORTER%% *}${SHORTER#* }"

  if [[ $ENVVAR = EPAGES_JAVA ]] ; then # treat EPAGES_JAVA specially
    JRE_HOME=$(jre_home)
    if [[ $JRE_HOME = $ENV ]] ; then
      ENV='$(jre_home)'
    elif [[ $(readlink -f "$JRE_HOME") = $(readlink -f "$ENV") ]] ; then
      ENV='$(jre_home)'
    fi
  fi
  [[ -n ${ENV//[0-9]/} ]] && ENV="\"$ENV\""
  ini-config.sh -f "$FROM" -k $ENVVAR_KEY -v "$ENV" -i
done

[[ -z $INPLACE ]] && cat "$FROM" || cat "$FROM" > "$ETC_DEFAULT"

rm -f "$FROM"*

exit 0
