#!/bin/env bash

BN=${0##*/}

usage() {
  echo "Usage:"
  echo "  $BN { --set | --restore } [ --perl-cgi ] [ --verbose ]"
  echo "Options:"
  echo "  --set     : set ePages SELinux context for web adapter files"
  echo "  --restore: set default SELinux context for web adapter files"
  echo "  --perl-cgi: set additionally SELinux context for Perl files"
  echo "  --verbose: verbose output"
  exit 1
}

PATH=$PATH:/sbin:/usr/sbin

unset SET RESTORE PERL_CGI VERBOSE
while [ $# -gt 0 ] ; do
  case $1 in
    --set) SET=1 ; shift ;;
    --restore) RESTORE=1 ; shift ;;
    --perl-cgi) PERL_CGI=$1 ; shift ;;
    --verbose) VERBOSE=$1 ; shift ;;
    *) usage ;;
  esac
done

[[ -n $SET && -n $RESTORE ]] && usage
[[ -z $SET && -z $RESTORE ]] && usage

[[ -z $VERBOSE ]] && exec 2>/dev/null

[[ -n $VERBOSE ]] && echo -n "-> chcon --version: "
chcon --version >/dev/null || exit 0
[[ -n $VERBOSE ]] && chcon --version | head -1
[[ -n $VERBOSE ]] && echo "-> getenforce: $(getenforce)"
[[ "$(getenforce | tr A-Z a-z)" = enforcing ]] || exit 0
[[ -n $VERBOSE ]] && echo "-> test -w /etc/passwd"
[ -w /etc/passwd ] || exit 0

if [[ -z $EPAGES ]] ; then
  echo "First execute: . /etc/default/epages6"
  exit 0
fi

run () {
  [[ -n $VERBOSE ]] && echo $@
  eval $@
}

is_local () {
  find "$@" -maxdepth 0 ! -fstype nfs 2>/dev/null | grep -q .
  return $?
}

# epages-mysqld

[[ -n $VERBOSE ]] && echo "=> epages-mysqld"

if [[ -n $SET ]] ; then
  [ -f /etc/init.d/mysql ] && run chcon system_u:object_r:textrel_shlib_t:s0 /etc/init.d/mysql
  IS_LINK=$(find /var/lib/mysql -maxdepth 0 -printf "%l" 2>/dev/null)
  if [[ -n $IS_LINK ]] ; then
    is_local /var/lib/mysql && run chcon -h system_u:object_r:mysqld_db_t:s0 /var/lib/mysql
    is_local $(dirname $IS_LINK) && run chcon -h system_u:object_r:var_lib_t:s0 $(dirname $IS_LINK)
  fi
else
  [ -f /etc/init.d/mysql ] && run restorecon /etc/init.d/mysql
  IS_LINK=$(find /var/lib/mysql -maxdepth 0 -printf "%l" 2>/dev/null)
  if [[ -n $IS_LINK ]] ; then
    is_local /var/lib/mysql && run restorecon /var/lib/mysql
    is_local $(dirname $IS_LINK) && run restorecon $(dirname $IS_LINK)
  fi
fi

# epages-webconf

[[ -n $VERBOSE ]] && echo "=> epages-webconf"

EPWEB=${EPAGES_WEBGROUP%%:*}
EPWEB=${EPWEB:-ep_web}
grep -q "^$EPWEB:" /etc/group || EPWEB=$(find $EPAGES_LOG -maxdepth 0 -printf "%g")

if [[ -n $SET ]] ; then
  is_local "$EPAGES_CONFIG"/WebInterface.conf && run chcon system_u:object_r:httpd_config_t:s0 "\$EPAGES_CONFIG"/WebInterface.conf
  is_local "$EPAGES_CONFIG"/ServerConfig.xml && run chcon system_u:object_r:httpd_config_t:s0 "\$EPAGES_CONFIG"/ServerConfig.xml
  if is_local "$EPAGES"/WebAdapter/ ; then
    run chcon system_u:object_r:httpd_modules_t:s0 "\$EPAGES"/WebAdapter/{lib64/,}\*.so
    run chcon system_u:object_r:bin_t:s0 "\$EPAGES"/WebAdapter/monitor.pl
  fi
  if is_local "$EPAGES_SHARED"/Monitor/ ; then
    run chcon system_u:object_r:bin_t:s0 "\$EPAGES_SHARED"/Monitor/spy.pl
    run chcon system_u:object_r:httpd_sys_content_t:s0 "\$EPAGES_SHARED"/Monitor/\*.tmpl
  fi
  touch "$EPAGES_LOG"/WebAdapter.log
  if chown ${EPAGES_APPUSER:-eprunapp}:$EPWEB "$EPAGES_LOG"/WebAdapter.log ; then
    is_local "$EPAGES_LOG"/WebAdapter.log && run chcon system_u:object_r:httpd_log_t:s0 "\$EPAGES_LOG"/WebAdapter.log
  else
    rm -f "$EPAGES_LOG"/WebAdapter.log
  fi
  if [[ $(ls -ldZ "$EPAGES_WEBROOT" | awk -F: '{print $3}') != httpd_sys_content_t ]] ; then
    is_local "$EPAGES_WEBROOT" && run chcon -h system_u:object_r:httpd_sys_content_t:s0 "\$EPAGES_WEBROOT"
    is_local "$EPAGES_WEBROOT"/ && run chcon -RL system_u:object_r:httpd_sys_content_t:s0 "\$EPAGES_WEBROOT"
  fi
else
  is_local "$EPAGES_CONFIG"/WebInterface.conf && run restorecon "\$EPAGES_CONFIG"/WebInterface.conf
  is_local "$EPAGES_CONFIG"/ServerConfig.xml && run restorecon "\$EPAGES_CONFIG"/ServerConfig.xml
  if is_local "$EPAGES"/WebAdapter/ ; then
    run restorecon "\$EPAGES"/WebAdapter/{lib64/,}*.so
    run restorecon "\$EPAGES"/WebAdapter/monitor.pl
  fi
  if is_local "$EPAGES_SHARED"/Monitor/ ; then
    run restorecon "\$EPAGES_SHARED"/Monitor/spy.pl
    run restorecon "\$EPAGES_SHARED"/Monitor/\*.tmpl
  fi
  is_local "$EPAGES_LOG"/WebAdapter.log && run restorecon "\$EPAGES_LOG"/WebAdapter.log
  is_local "$EPAGES_WEBROOT"/ && run restorecon -R "\$EPAGES_WEBROOT" "\$EPAGES_WEBROOT"/\*
fi

# epages-perl

if [[ -n $PERL_CGI ]] ; then

  if [[ -n $SET ]] ; then
    if [[ $(ls -ldZ "$EPAGES_CARTRIDGES" | awk -F: '{print $3}') != usr_t ]] ; then
      is_local "$EPAGES_CARTRIDGES" && run chcon -h system_u:object_r:usr_t:s0 "\$EPAGES_CARTRIDGES"
      is_local "$EPAGES_CARTRIDGES"/ && run chcon -RL system_u:object_r:usr_t:s0 "\$EPAGES_CARTRIDGES"
    fi

    if is_local "$EPAGES_PERL"/bin ; then
      run chcon -RL system_u:object_r:bin_t:s0 "\$EPAGES_PERL"/bin
      find "$EPAGES_PERL"/bin -name "*.so*" -exec chcon system_u:object_r:lib_t:s0 {} \;
      find "$EPAGES_PERL"/bin/libMagick*.so.* -exec chcon system_u:object_r:textrel_shlib_t:s0 {} \;
    fi

    if is_local "$EPAGES_PERL"/lib ; then
      if [[ $(ls -ldZ "$EPAGES_PERL"/lib | awk -F: '{print $3}') != usr_t ]] ; then
        run chcon -RL system_u:object_r:usr_t:s0 "\$EPAGES_PERL"/lib
      fi
      find "$EPAGES_PERL"/lib* -name "*.so*" -exec chcon system_u:object_r:lib_t:s0 {} \;
      if [ -f "$EPAGES_PERL"/lib/site_perl/linux/auto/Math/Pari/Pari.so ] ; then
        run chcon system_u:object_r:textrel_shlib_t:s0 "\$EPAGES_PERL"/lib/site_perl/linux/auto/Math/Pari/Pari.so
      fi
    fi

    if is_local "$EPAGES_LOG"/ ; then
      for i in debug error ; do
        touch "$EPAGES_LOG"/$i.log
        if chown ${EPAGES_APPUSER:-eprunapp}:$EPWEB "$EPAGES_LOG"/$i.log ; then
          run chcon system_u:object_r:httpd_log_t:s0 "\$EPAGES_LOG"/$i.log
        else
          rm -f "$EPAGES_LOG"/$i.log
        fi
      done
    fi

  else
    is_local "$EPAGES_CARTRIDGES"/ && run restorecon -R "\$EPAGES_CARTRIDGES" "\$EPAGES_CARTRIDGES"/\*
    is_local "$EPAGES_PERL"/ && run restorecon -R "\$EPAGES_PERL" "\$EPAGES_PERL"/\*
    is_local "$EPAGES_LOG"/ && run restorecon -R "\$EPAGES_LOG" "\$EPAGES_LOG"/\*
  fi

fi
